from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from .. import types
from ..types import UNSET, Unset

T = TypeVar("T", bound="PatchedUser")


@_attrs_define
class PatchedUser:
    """Adds nested create feature

    Attributes:
        id (int | Unset):
        username (str | Unset): Required. 150 characters or fewer. Letters, digits and @/./+/-/_ only.
        first_name (str | Unset):
        last_name (str | Unset):
        display_name (str | Unset):
        is_staff (bool | Unset): Designates whether the user can log into this admin site.
        is_superuser (bool | Unset): Designates that this user has all permissions without explicitly assigning them.
        is_active (bool | Unset): Designates whether this user should be treated as active. Unselect this instead of
            deleting accounts.
    """

    id: int | Unset = UNSET
    username: str | Unset = UNSET
    first_name: str | Unset = UNSET
    last_name: str | Unset = UNSET
    display_name: str | Unset = UNSET
    is_staff: bool | Unset = UNSET
    is_superuser: bool | Unset = UNSET
    is_active: bool | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        id = self.id

        username = self.username

        first_name = self.first_name

        last_name = self.last_name

        display_name = self.display_name

        is_staff = self.is_staff

        is_superuser = self.is_superuser

        is_active = self.is_active

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if id is not UNSET:
            field_dict["id"] = id
        if username is not UNSET:
            field_dict["username"] = username
        if first_name is not UNSET:
            field_dict["first_name"] = first_name
        if last_name is not UNSET:
            field_dict["last_name"] = last_name
        if display_name is not UNSET:
            field_dict["display_name"] = display_name
        if is_staff is not UNSET:
            field_dict["is_staff"] = is_staff
        if is_superuser is not UNSET:
            field_dict["is_superuser"] = is_superuser
        if is_active is not UNSET:
            field_dict["is_active"] = is_active

        return field_dict

    def to_multipart(self) -> types.RequestFiles:
        files: types.RequestFiles = []

        if not isinstance(self.id, Unset):
            files.append(("id", (None, str(self.id).encode(), "text/plain")))

        if not isinstance(self.username, Unset):
            files.append(("username", (None, str(self.username).encode(), "text/plain")))

        if not isinstance(self.first_name, Unset):
            files.append(("first_name", (None, str(self.first_name).encode(), "text/plain")))

        if not isinstance(self.last_name, Unset):
            files.append(("last_name", (None, str(self.last_name).encode(), "text/plain")))

        if not isinstance(self.display_name, Unset):
            files.append(("display_name", (None, str(self.display_name).encode(), "text/plain")))

        if not isinstance(self.is_staff, Unset):
            files.append(("is_staff", (None, str(self.is_staff).encode(), "text/plain")))

        if not isinstance(self.is_superuser, Unset):
            files.append(("is_superuser", (None, str(self.is_superuser).encode(), "text/plain")))

        if not isinstance(self.is_active, Unset):
            files.append(("is_active", (None, str(self.is_active).encode(), "text/plain")))

        for prop_name, prop in self.additional_properties.items():
            files.append((prop_name, (None, str(prop).encode(), "text/plain")))

        return files

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        id = d.pop("id", UNSET)

        username = d.pop("username", UNSET)

        first_name = d.pop("first_name", UNSET)

        last_name = d.pop("last_name", UNSET)

        display_name = d.pop("display_name", UNSET)

        is_staff = d.pop("is_staff", UNSET)

        is_superuser = d.pop("is_superuser", UNSET)

        is_active = d.pop("is_active", UNSET)

        patched_user = cls(
            id=id,
            username=username,
            first_name=first_name,
            last_name=last_name,
            display_name=display_name,
            is_staff=is_staff,
            is_superuser=is_superuser,
            is_active=is_active,
        )

        patched_user.additional_properties = d
        return patched_user

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class ApiAutomationListTypeItem(str, Enum):
    DESCRIPTION_REPLACE = "DESCRIPTION_REPLACE"
    FOOD_ALIAS = "FOOD_ALIAS"
    FOOD_REPLACE = "FOOD_REPLACE"
    INSTRUCTION_REPLACE = "INSTRUCTION_REPLACE"
    KEYWORD_ALIAS = "KEYWORD_ALIAS"
    NAME_REPLACE = "NAME_REPLACE"
    NEVER_UNIT = "NEVER_UNIT"
    TRANSPOSE_WORDS = "TRANSPOSE_WORDS"
    UNIT_ALIAS = "UNIT_ALIAS"
    UNIT_REPLACE = "UNIT_REPLACE"

    def __str__(self) -> str:
        return str(self.value)



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from .. import types
from ..types import UNSET, Unset

T = TypeVar("T", bound="SupermarketCategory")


@_attrs_define
class SupermarketCategory:
    """Moves `UniqueValidator`'s from the validation stage to the save stage.
    It solves the problem with nested validation for unique fields on update.

    If you want more details, you can read related issues and articles:
    https://github.com/beda-software/drf-writable-nested/issues/1
    http://www.django-rest-framework.org/api-guide/validators/#updating-nested-serializers

    Example of usage:
    ```
        class Child(models.Model):
        field = models.CharField(unique=True)


    class Parent(models.Model):
        child = models.ForeignKey('Child')


    class ChildSerializer(UniqueFieldsMixin, serializers.ModelSerializer):
        class Meta:
            model = Child


    class ParentSerializer(NestedUpdateMixin, serializers.ModelSerializer):
        child = ChildSerializer()

        class Meta:
            model = Parent
    ```

    Note: `UniqueFieldsMixin` must be applied only on the serializer
    which has unique fields.

    Note: When you are using both mixins
    (`UniqueFieldsMixin` and `NestedCreateMixin` or `NestedUpdateMixin`)
    you should put `UniqueFieldsMixin` ahead.

        Attributes:
            name (str):
            id (int | Unset):
            description (None | str | Unset):
            open_data_slug (None | str | Unset):
    """

    name: str
    id: int | Unset = UNSET
    description: None | str | Unset = UNSET
    open_data_slug: None | str | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        name = self.name

        id = self.id

        description: None | str | Unset
        if isinstance(self.description, Unset):
            description = UNSET
        else:
            description = self.description

        open_data_slug: None | str | Unset
        if isinstance(self.open_data_slug, Unset):
            open_data_slug = UNSET
        else:
            open_data_slug = self.open_data_slug

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "name": name,
            }
        )
        if id is not UNSET:
            field_dict["id"] = id
        if description is not UNSET:
            field_dict["description"] = description
        if open_data_slug is not UNSET:
            field_dict["open_data_slug"] = open_data_slug

        return field_dict

    def to_multipart(self) -> types.RequestFiles:
        files: types.RequestFiles = []

        files.append(("name", (None, str(self.name).encode(), "text/plain")))

        if not isinstance(self.id, Unset):
            files.append(("id", (None, str(self.id).encode(), "text/plain")))

        if not isinstance(self.description, Unset):
            if isinstance(self.description, str):
                files.append(("description", (None, str(self.description).encode(), "text/plain")))
            else:
                files.append(("description", (None, str(self.description).encode(), "text/plain")))

        if not isinstance(self.open_data_slug, Unset):
            if isinstance(self.open_data_slug, str):
                files.append(("open_data_slug", (None, str(self.open_data_slug).encode(), "text/plain")))
            else:
                files.append(("open_data_slug", (None, str(self.open_data_slug).encode(), "text/plain")))

        for prop_name, prop in self.additional_properties.items():
            files.append((prop_name, (None, str(prop).encode(), "text/plain")))

        return files

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        name = d.pop("name")

        id = d.pop("id", UNSET)

        def _parse_description(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        description = _parse_description(d.pop("description", UNSET))

        def _parse_open_data_slug(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        open_data_slug = _parse_open_data_slug(d.pop("open_data_slug", UNSET))

        supermarket_category = cls(
            name=name,
            id=id,
            description=description,
            open_data_slug=open_data_slug,
        )

        supermarket_category.additional_properties = d
        return supermarket_category

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

import json
from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from .. import types
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.food_inherit_field import FoodInheritField
    from ..models.food_simple import FoodSimple
    from ..models.property_ import Property
    from ..models.recipe_simple import RecipeSimple
    from ..models.shopping_list import ShoppingList
    from ..models.supermarket_category import SupermarketCategory
    from ..models.unit import Unit


T = TypeVar("T", bound="PatchedFood")


@_attrs_define
class PatchedFood:
    """Moves `UniqueValidator`'s from the validation stage to the save stage.
    It solves the problem with nested validation for unique fields on update.

    If you want more details, you can read related issues and articles:
    https://github.com/beda-software/drf-writable-nested/issues/1
    http://www.django-rest-framework.org/api-guide/validators/#updating-nested-serializers

    Example of usage:
    ```
        class Child(models.Model):
        field = models.CharField(unique=True)


    class Parent(models.Model):
        child = models.ForeignKey('Child')


    class ChildSerializer(UniqueFieldsMixin, serializers.ModelSerializer):
        class Meta:
            model = Child


    class ParentSerializer(NestedUpdateMixin, serializers.ModelSerializer):
        child = ChildSerializer()

        class Meta:
            model = Parent
    ```

    Note: `UniqueFieldsMixin` must be applied only on the serializer
    which has unique fields.

    Note: When you are using both mixins
    (`UniqueFieldsMixin` and `NestedCreateMixin` or `NestedUpdateMixin`)
    you should put `UniqueFieldsMixin` ahead.

        Attributes:
            id (int | Unset):
            name (str | Unset):
            plural_name (None | str | Unset):
            description (str | Unset):
            shopping (str | Unset):
            recipe (None | RecipeSimple | Unset):
            url (None | str | Unset):
            properties (list[Property] | None | Unset):
            properties_food_amount (float | Unset):
            properties_food_unit (None | Unit | Unset):
            fdc_id (int | None | Unset):
            food_onhand (bool | None | Unset):
            supermarket_category (None | SupermarketCategory | Unset):
            parent (int | Unset):
            numchild (int | Unset):
            inherit_fields (list[FoodInheritField] | None | Unset):
            full_name (str | Unset): Returns a string representation of a tree node and it's ancestors,
                e.g. 'Cuisine > Asian > Chinese > Catonese'.
            ignore_shopping (bool | Unset):
            substitute (list[FoodSimple] | None | Unset):
            substitute_siblings (bool | Unset):
            substitute_children (bool | Unset):
            substitute_onhand (bool | Unset):
            child_inherit_fields (list[FoodInheritField] | None | Unset):
            open_data_slug (None | str | Unset):
            shopping_lists (list[ShoppingList] | Unset):
    """

    id: int | Unset = UNSET
    name: str | Unset = UNSET
    plural_name: None | str | Unset = UNSET
    description: str | Unset = UNSET
    shopping: str | Unset = UNSET
    recipe: None | RecipeSimple | Unset = UNSET
    url: None | str | Unset = UNSET
    properties: list[Property] | None | Unset = UNSET
    properties_food_amount: float | Unset = UNSET
    properties_food_unit: None | Unit | Unset = UNSET
    fdc_id: int | None | Unset = UNSET
    food_onhand: bool | None | Unset = UNSET
    supermarket_category: None | SupermarketCategory | Unset = UNSET
    parent: int | Unset = UNSET
    numchild: int | Unset = UNSET
    inherit_fields: list[FoodInheritField] | None | Unset = UNSET
    full_name: str | Unset = UNSET
    ignore_shopping: bool | Unset = UNSET
    substitute: list[FoodSimple] | None | Unset = UNSET
    substitute_siblings: bool | Unset = UNSET
    substitute_children: bool | Unset = UNSET
    substitute_onhand: bool | Unset = UNSET
    child_inherit_fields: list[FoodInheritField] | None | Unset = UNSET
    open_data_slug: None | str | Unset = UNSET
    shopping_lists: list[ShoppingList] | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        from ..models.recipe_simple import RecipeSimple
        from ..models.supermarket_category import SupermarketCategory
        from ..models.unit import Unit

        id = self.id

        name = self.name

        plural_name: None | str | Unset
        if isinstance(self.plural_name, Unset):
            plural_name = UNSET
        else:
            plural_name = self.plural_name

        description = self.description

        shopping = self.shopping

        recipe: dict[str, Any] | None | Unset
        if isinstance(self.recipe, Unset):
            recipe = UNSET
        elif isinstance(self.recipe, RecipeSimple):
            recipe = self.recipe.to_dict()
        else:
            recipe = self.recipe

        url: None | str | Unset
        if isinstance(self.url, Unset):
            url = UNSET
        else:
            url = self.url

        properties: list[dict[str, Any]] | None | Unset
        if isinstance(self.properties, Unset):
            properties = UNSET
        elif isinstance(self.properties, list):
            properties = []
            for properties_type_0_item_data in self.properties:
                properties_type_0_item = properties_type_0_item_data.to_dict()
                properties.append(properties_type_0_item)

        else:
            properties = self.properties

        properties_food_amount = self.properties_food_amount

        properties_food_unit: dict[str, Any] | None | Unset
        if isinstance(self.properties_food_unit, Unset):
            properties_food_unit = UNSET
        elif isinstance(self.properties_food_unit, Unit):
            properties_food_unit = self.properties_food_unit.to_dict()
        else:
            properties_food_unit = self.properties_food_unit

        fdc_id: int | None | Unset
        if isinstance(self.fdc_id, Unset):
            fdc_id = UNSET
        else:
            fdc_id = self.fdc_id

        food_onhand: bool | None | Unset
        if isinstance(self.food_onhand, Unset):
            food_onhand = UNSET
        else:
            food_onhand = self.food_onhand

        supermarket_category: dict[str, Any] | None | Unset
        if isinstance(self.supermarket_category, Unset):
            supermarket_category = UNSET
        elif isinstance(self.supermarket_category, SupermarketCategory):
            supermarket_category = self.supermarket_category.to_dict()
        else:
            supermarket_category = self.supermarket_category

        parent = self.parent

        numchild = self.numchild

        inherit_fields: list[dict[str, Any]] | None | Unset
        if isinstance(self.inherit_fields, Unset):
            inherit_fields = UNSET
        elif isinstance(self.inherit_fields, list):
            inherit_fields = []
            for inherit_fields_type_0_item_data in self.inherit_fields:
                inherit_fields_type_0_item = inherit_fields_type_0_item_data.to_dict()
                inherit_fields.append(inherit_fields_type_0_item)

        else:
            inherit_fields = self.inherit_fields

        full_name = self.full_name

        ignore_shopping = self.ignore_shopping

        substitute: list[dict[str, Any]] | None | Unset
        if isinstance(self.substitute, Unset):
            substitute = UNSET
        elif isinstance(self.substitute, list):
            substitute = []
            for substitute_type_0_item_data in self.substitute:
                substitute_type_0_item = substitute_type_0_item_data.to_dict()
                substitute.append(substitute_type_0_item)

        else:
            substitute = self.substitute

        substitute_siblings = self.substitute_siblings

        substitute_children = self.substitute_children

        substitute_onhand = self.substitute_onhand

        child_inherit_fields: list[dict[str, Any]] | None | Unset
        if isinstance(self.child_inherit_fields, Unset):
            child_inherit_fields = UNSET
        elif isinstance(self.child_inherit_fields, list):
            child_inherit_fields = []
            for child_inherit_fields_type_0_item_data in self.child_inherit_fields:
                child_inherit_fields_type_0_item = child_inherit_fields_type_0_item_data.to_dict()
                child_inherit_fields.append(child_inherit_fields_type_0_item)

        else:
            child_inherit_fields = self.child_inherit_fields

        open_data_slug: None | str | Unset
        if isinstance(self.open_data_slug, Unset):
            open_data_slug = UNSET
        else:
            open_data_slug = self.open_data_slug

        shopping_lists: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.shopping_lists, Unset):
            shopping_lists = []
            for shopping_lists_item_data in self.shopping_lists:
                shopping_lists_item = shopping_lists_item_data.to_dict()
                shopping_lists.append(shopping_lists_item)

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if id is not UNSET:
            field_dict["id"] = id
        if name is not UNSET:
            field_dict["name"] = name
        if plural_name is not UNSET:
            field_dict["plural_name"] = plural_name
        if description is not UNSET:
            field_dict["description"] = description
        if shopping is not UNSET:
            field_dict["shopping"] = shopping
        if recipe is not UNSET:
            field_dict["recipe"] = recipe
        if url is not UNSET:
            field_dict["url"] = url
        if properties is not UNSET:
            field_dict["properties"] = properties
        if properties_food_amount is not UNSET:
            field_dict["properties_food_amount"] = properties_food_amount
        if properties_food_unit is not UNSET:
            field_dict["properties_food_unit"] = properties_food_unit
        if fdc_id is not UNSET:
            field_dict["fdc_id"] = fdc_id
        if food_onhand is not UNSET:
            field_dict["food_onhand"] = food_onhand
        if supermarket_category is not UNSET:
            field_dict["supermarket_category"] = supermarket_category
        if parent is not UNSET:
            field_dict["parent"] = parent
        if numchild is not UNSET:
            field_dict["numchild"] = numchild
        if inherit_fields is not UNSET:
            field_dict["inherit_fields"] = inherit_fields
        if full_name is not UNSET:
            field_dict["full_name"] = full_name
        if ignore_shopping is not UNSET:
            field_dict["ignore_shopping"] = ignore_shopping
        if substitute is not UNSET:
            field_dict["substitute"] = substitute
        if substitute_siblings is not UNSET:
            field_dict["substitute_siblings"] = substitute_siblings
        if substitute_children is not UNSET:
            field_dict["substitute_children"] = substitute_children
        if substitute_onhand is not UNSET:
            field_dict["substitute_onhand"] = substitute_onhand
        if child_inherit_fields is not UNSET:
            field_dict["child_inherit_fields"] = child_inherit_fields
        if open_data_slug is not UNSET:
            field_dict["open_data_slug"] = open_data_slug
        if shopping_lists is not UNSET:
            field_dict["shopping_lists"] = shopping_lists

        return field_dict

    def to_multipart(self) -> types.RequestFiles:
        files: types.RequestFiles = []

        if not isinstance(self.id, Unset):
            files.append(("id", (None, str(self.id).encode(), "text/plain")))

        if not isinstance(self.name, Unset):
            files.append(("name", (None, str(self.name).encode(), "text/plain")))

        if not isinstance(self.plural_name, Unset):
            if isinstance(self.plural_name, str):
                files.append(("plural_name", (None, str(self.plural_name).encode(), "text/plain")))
            else:
                files.append(("plural_name", (None, str(self.plural_name).encode(), "text/plain")))

        if not isinstance(self.description, Unset):
            files.append(("description", (None, str(self.description).encode(), "text/plain")))

        if not isinstance(self.shopping, Unset):
            files.append(("shopping", (None, str(self.shopping).encode(), "text/plain")))

        if not isinstance(self.recipe, Unset):
            if self.recipe is None:
                files.append(("recipe", (None, str(self.recipe).encode(), "text/plain")))
            else:
                files.append(("recipe", (None, json.dumps(self.recipe.to_dict()).encode(), "application/json")))

        if not isinstance(self.url, Unset):
            if isinstance(self.url, str):
                files.append(("url", (None, str(self.url).encode(), "text/plain")))
            else:
                files.append(("url", (None, str(self.url).encode(), "text/plain")))

        if not isinstance(self.properties, Unset):
            if isinstance(self.properties, list):
                for properties_type_0_item_element in self.properties:
                    files.append(
                        (
                            "properties",
                            (None, json.dumps(properties_type_0_item_element.to_dict()).encode(), "application/json"),
                        )
                    )
            else:
                files.append(("properties", (None, str(self.properties).encode(), "text/plain")))

        if not isinstance(self.properties_food_amount, Unset):
            files.append(("properties_food_amount", (None, str(self.properties_food_amount).encode(), "text/plain")))

        if not isinstance(self.properties_food_unit, Unset):
            if self.properties_food_unit is None:
                files.append(("properties_food_unit", (None, str(self.properties_food_unit).encode(), "text/plain")))
            else:
                files.append(
                    (
                        "properties_food_unit",
                        (None, json.dumps(self.properties_food_unit.to_dict()).encode(), "application/json"),
                    )
                )

        if not isinstance(self.fdc_id, Unset):
            if isinstance(self.fdc_id, int):
                files.append(("fdc_id", (None, str(self.fdc_id).encode(), "text/plain")))
            else:
                files.append(("fdc_id", (None, str(self.fdc_id).encode(), "text/plain")))

        if not isinstance(self.food_onhand, Unset):
            if isinstance(self.food_onhand, bool):
                files.append(("food_onhand", (None, str(self.food_onhand).encode(), "text/plain")))
            else:
                files.append(("food_onhand", (None, str(self.food_onhand).encode(), "text/plain")))

        if not isinstance(self.supermarket_category, Unset):
            if self.supermarket_category is None:
                files.append(("supermarket_category", (None, str(self.supermarket_category).encode(), "text/plain")))
            else:
                files.append(
                    (
                        "supermarket_category",
                        (None, json.dumps(self.supermarket_category.to_dict()).encode(), "application/json"),
                    )
                )

        if not isinstance(self.parent, Unset):
            files.append(("parent", (None, str(self.parent).encode(), "text/plain")))

        if not isinstance(self.numchild, Unset):
            files.append(("numchild", (None, str(self.numchild).encode(), "text/plain")))

        if not isinstance(self.inherit_fields, Unset):
            if isinstance(self.inherit_fields, list):
                for inherit_fields_type_0_item_element in self.inherit_fields:
                    files.append(
                        (
                            "inherit_fields",
                            (
                                None,
                                json.dumps(inherit_fields_type_0_item_element.to_dict()).encode(),
                                "application/json",
                            ),
                        )
                    )
            else:
                files.append(("inherit_fields", (None, str(self.inherit_fields).encode(), "text/plain")))

        if not isinstance(self.full_name, Unset):
            files.append(("full_name", (None, str(self.full_name).encode(), "text/plain")))

        if not isinstance(self.ignore_shopping, Unset):
            files.append(("ignore_shopping", (None, str(self.ignore_shopping).encode(), "text/plain")))

        if not isinstance(self.substitute, Unset):
            if isinstance(self.substitute, list):
                for substitute_type_0_item_element in self.substitute:
                    files.append(
                        (
                            "substitute",
                            (None, json.dumps(substitute_type_0_item_element.to_dict()).encode(), "application/json"),
                        )
                    )
            else:
                files.append(("substitute", (None, str(self.substitute).encode(), "text/plain")))

        if not isinstance(self.substitute_siblings, Unset):
            files.append(("substitute_siblings", (None, str(self.substitute_siblings).encode(), "text/plain")))

        if not isinstance(self.substitute_children, Unset):
            files.append(("substitute_children", (None, str(self.substitute_children).encode(), "text/plain")))

        if not isinstance(self.substitute_onhand, Unset):
            files.append(("substitute_onhand", (None, str(self.substitute_onhand).encode(), "text/plain")))

        if not isinstance(self.child_inherit_fields, Unset):
            if isinstance(self.child_inherit_fields, list):
                for child_inherit_fields_type_0_item_element in self.child_inherit_fields:
                    files.append(
                        (
                            "child_inherit_fields",
                            (
                                None,
                                json.dumps(child_inherit_fields_type_0_item_element.to_dict()).encode(),
                                "application/json",
                            ),
                        )
                    )
            else:
                files.append(("child_inherit_fields", (None, str(self.child_inherit_fields).encode(), "text/plain")))

        if not isinstance(self.open_data_slug, Unset):
            if isinstance(self.open_data_slug, str):
                files.append(("open_data_slug", (None, str(self.open_data_slug).encode(), "text/plain")))
            else:
                files.append(("open_data_slug", (None, str(self.open_data_slug).encode(), "text/plain")))

        if not isinstance(self.shopping_lists, Unset):
            for shopping_lists_item_element in self.shopping_lists:
                files.append(
                    (
                        "shopping_lists",
                        (None, json.dumps(shopping_lists_item_element.to_dict()).encode(), "application/json"),
                    )
                )

        for prop_name, prop in self.additional_properties.items():
            files.append((prop_name, (None, str(prop).encode(), "text/plain")))

        return files

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.food_inherit_field import FoodInheritField
        from ..models.food_simple import FoodSimple
        from ..models.property_ import Property
        from ..models.recipe_simple import RecipeSimple
        from ..models.shopping_list import ShoppingList
        from ..models.supermarket_category import SupermarketCategory
        from ..models.unit import Unit

        d = dict(src_dict)
        id = d.pop("id", UNSET)

        name = d.pop("name", UNSET)

        def _parse_plural_name(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        plural_name = _parse_plural_name(d.pop("plural_name", UNSET))

        description = d.pop("description", UNSET)

        shopping = d.pop("shopping", UNSET)

        def _parse_recipe(data: object) -> None | RecipeSimple | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                recipe_type_1 = RecipeSimple.from_dict(data)

                return recipe_type_1
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(None | RecipeSimple | Unset, data)

        recipe = _parse_recipe(d.pop("recipe", UNSET))

        def _parse_url(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        url = _parse_url(d.pop("url", UNSET))

        def _parse_properties(data: object) -> list[Property] | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                properties_type_0 = []
                _properties_type_0 = data
                for properties_type_0_item_data in _properties_type_0:
                    properties_type_0_item = Property.from_dict(properties_type_0_item_data)

                    properties_type_0.append(properties_type_0_item)

                return properties_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(list[Property] | None | Unset, data)

        properties = _parse_properties(d.pop("properties", UNSET))

        properties_food_amount = d.pop("properties_food_amount", UNSET)

        def _parse_properties_food_unit(data: object) -> None | Unit | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                properties_food_unit_type_1 = Unit.from_dict(data)

                return properties_food_unit_type_1
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(None | Unit | Unset, data)

        properties_food_unit = _parse_properties_food_unit(d.pop("properties_food_unit", UNSET))

        def _parse_fdc_id(data: object) -> int | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(int | None | Unset, data)

        fdc_id = _parse_fdc_id(d.pop("fdc_id", UNSET))

        def _parse_food_onhand(data: object) -> bool | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(bool | None | Unset, data)

        food_onhand = _parse_food_onhand(d.pop("food_onhand", UNSET))

        def _parse_supermarket_category(data: object) -> None | SupermarketCategory | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                supermarket_category_type_1 = SupermarketCategory.from_dict(data)

                return supermarket_category_type_1
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(None | SupermarketCategory | Unset, data)

        supermarket_category = _parse_supermarket_category(d.pop("supermarket_category", UNSET))

        parent = d.pop("parent", UNSET)

        numchild = d.pop("numchild", UNSET)

        def _parse_inherit_fields(data: object) -> list[FoodInheritField] | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                inherit_fields_type_0 = []
                _inherit_fields_type_0 = data
                for inherit_fields_type_0_item_data in _inherit_fields_type_0:
                    inherit_fields_type_0_item = FoodInheritField.from_dict(inherit_fields_type_0_item_data)

                    inherit_fields_type_0.append(inherit_fields_type_0_item)

                return inherit_fields_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(list[FoodInheritField] | None | Unset, data)

        inherit_fields = _parse_inherit_fields(d.pop("inherit_fields", UNSET))

        full_name = d.pop("full_name", UNSET)

        ignore_shopping = d.pop("ignore_shopping", UNSET)

        def _parse_substitute(data: object) -> list[FoodSimple] | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                substitute_type_0 = []
                _substitute_type_0 = data
                for substitute_type_0_item_data in _substitute_type_0:
                    substitute_type_0_item = FoodSimple.from_dict(substitute_type_0_item_data)

                    substitute_type_0.append(substitute_type_0_item)

                return substitute_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(list[FoodSimple] | None | Unset, data)

        substitute = _parse_substitute(d.pop("substitute", UNSET))

        substitute_siblings = d.pop("substitute_siblings", UNSET)

        substitute_children = d.pop("substitute_children", UNSET)

        substitute_onhand = d.pop("substitute_onhand", UNSET)

        def _parse_child_inherit_fields(data: object) -> list[FoodInheritField] | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                child_inherit_fields_type_0 = []
                _child_inherit_fields_type_0 = data
                for child_inherit_fields_type_0_item_data in _child_inherit_fields_type_0:
                    child_inherit_fields_type_0_item = FoodInheritField.from_dict(child_inherit_fields_type_0_item_data)

                    child_inherit_fields_type_0.append(child_inherit_fields_type_0_item)

                return child_inherit_fields_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(list[FoodInheritField] | None | Unset, data)

        child_inherit_fields = _parse_child_inherit_fields(d.pop("child_inherit_fields", UNSET))

        def _parse_open_data_slug(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        open_data_slug = _parse_open_data_slug(d.pop("open_data_slug", UNSET))

        _shopping_lists = d.pop("shopping_lists", UNSET)
        shopping_lists: list[ShoppingList] | Unset = UNSET
        if _shopping_lists is not UNSET:
            shopping_lists = []
            for shopping_lists_item_data in _shopping_lists:
                shopping_lists_item = ShoppingList.from_dict(shopping_lists_item_data)

                shopping_lists.append(shopping_lists_item)

        patched_food = cls(
            id=id,
            name=name,
            plural_name=plural_name,
            description=description,
            shopping=shopping,
            recipe=recipe,
            url=url,
            properties=properties,
            properties_food_amount=properties_food_amount,
            properties_food_unit=properties_food_unit,
            fdc_id=fdc_id,
            food_onhand=food_onhand,
            supermarket_category=supermarket_category,
            parent=parent,
            numchild=numchild,
            inherit_fields=inherit_fields,
            full_name=full_name,
            ignore_shopping=ignore_shopping,
            substitute=substitute,
            substitute_siblings=substitute_siblings,
            substitute_children=substitute_children,
            substitute_onhand=substitute_onhand,
            child_inherit_fields=child_inherit_fields,
            open_data_slug=open_data_slug,
            shopping_lists=shopping_lists,
        )

        patched_food.additional_properties = d
        return patched_food

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

import json
from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from .. import types
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.custom_filter import CustomFilter
    from ..models.user import User


T = TypeVar("T", bound="RecipeBook")


@_attrs_define
class RecipeBook:
    """Adds nested create feature

    Attributes:
        name (str):
        shared (list[User]):
        created_by (User): Adds nested create feature
        id (int | Unset):
        description (str | Unset):
        filter_ (CustomFilter | None | Unset):
        order (int | Unset):
    """

    name: str
    shared: list[User]
    created_by: User
    id: int | Unset = UNSET
    description: str | Unset = UNSET
    filter_: CustomFilter | None | Unset = UNSET
    order: int | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        from ..models.custom_filter import CustomFilter

        name = self.name

        shared = []
        for shared_item_data in self.shared:
            shared_item = shared_item_data.to_dict()
            shared.append(shared_item)

        created_by = self.created_by.to_dict()

        id = self.id

        description = self.description

        filter_: dict[str, Any] | None | Unset
        if isinstance(self.filter_, Unset):
            filter_ = UNSET
        elif isinstance(self.filter_, CustomFilter):
            filter_ = self.filter_.to_dict()
        else:
            filter_ = self.filter_

        order = self.order

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "name": name,
                "shared": shared,
                "created_by": created_by,
            }
        )
        if id is not UNSET:
            field_dict["id"] = id
        if description is not UNSET:
            field_dict["description"] = description
        if filter_ is not UNSET:
            field_dict["filter"] = filter_
        if order is not UNSET:
            field_dict["order"] = order

        return field_dict

    def to_multipart(self) -> types.RequestFiles:
        files: types.RequestFiles = []

        files.append(("name", (None, str(self.name).encode(), "text/plain")))

        for shared_item_element in self.shared:
            files.append(("shared", (None, json.dumps(shared_item_element.to_dict()).encode(), "application/json")))

        files.append(("created_by", (None, json.dumps(self.created_by.to_dict()).encode(), "application/json")))

        if not isinstance(self.id, Unset):
            files.append(("id", (None, str(self.id).encode(), "text/plain")))

        if not isinstance(self.description, Unset):
            files.append(("description", (None, str(self.description).encode(), "text/plain")))

        if not isinstance(self.filter_, Unset):
            if self.filter_ is None:
                files.append(("filter", (None, str(self.filter_).encode(), "text/plain")))
            else:
                files.append(("filter", (None, json.dumps(self.filter_.to_dict()).encode(), "application/json")))

        if not isinstance(self.order, Unset):
            files.append(("order", (None, str(self.order).encode(), "text/plain")))

        for prop_name, prop in self.additional_properties.items():
            files.append((prop_name, (None, str(prop).encode(), "text/plain")))

        return files

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.custom_filter import CustomFilter
        from ..models.user import User

        d = dict(src_dict)
        name = d.pop("name")

        shared = []
        _shared = d.pop("shared")
        for shared_item_data in _shared:
            shared_item = User.from_dict(shared_item_data)

            shared.append(shared_item)

        created_by = User.from_dict(d.pop("created_by"))

        id = d.pop("id", UNSET)

        description = d.pop("description", UNSET)

        def _parse_filter_(data: object) -> CustomFilter | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                filter_type_1 = CustomFilter.from_dict(data)

                return filter_type_1
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(CustomFilter | None | Unset, data)

        filter_ = _parse_filter_(d.pop("filter", UNSET))

        order = d.pop("order", UNSET)

        recipe_book = cls(
            name=name,
            shared=shared,
            created_by=created_by,
            id=id,
            description=description,
            filter_=filter_,
            order=order,
        )

        recipe_book.additional_properties = d
        return recipe_book

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

import json
from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from .. import types
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.ingredient import Ingredient
    from ..models.user_file_view import UserFileView


T = TypeVar("T", bound="PatchedStep")


@_attrs_define
class PatchedStep:
    """Adds nested create feature

    Attributes:
        id (int | Unset):
        name (str | Unset):
        instruction (str | Unset):
        ingredients (list[Ingredient] | Unset):
        instructions_markdown (str | Unset):
        time (int | Unset):
        order (int | Unset):
        show_as_header (bool | Unset):
        file (None | Unset | UserFileView):
        step_recipe (int | None | Unset):
        step_recipe_data (Any | Unset):
        numrecipe (int | Unset):
        show_ingredients_table (bool | Unset):
    """

    id: int | Unset = UNSET
    name: str | Unset = UNSET
    instruction: str | Unset = UNSET
    ingredients: list[Ingredient] | Unset = UNSET
    instructions_markdown: str | Unset = UNSET
    time: int | Unset = UNSET
    order: int | Unset = UNSET
    show_as_header: bool | Unset = UNSET
    file: None | Unset | UserFileView = UNSET
    step_recipe: int | None | Unset = UNSET
    step_recipe_data: Any | Unset = UNSET
    numrecipe: int | Unset = UNSET
    show_ingredients_table: bool | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        from ..models.user_file_view import UserFileView

        id = self.id

        name = self.name

        instruction = self.instruction

        ingredients: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.ingredients, Unset):
            ingredients = []
            for ingredients_item_data in self.ingredients:
                ingredients_item = ingredients_item_data.to_dict()
                ingredients.append(ingredients_item)

        instructions_markdown = self.instructions_markdown

        time = self.time

        order = self.order

        show_as_header = self.show_as_header

        file: dict[str, Any] | None | Unset
        if isinstance(self.file, Unset):
            file = UNSET
        elif isinstance(self.file, UserFileView):
            file = self.file.to_dict()
        else:
            file = self.file

        step_recipe: int | None | Unset
        if isinstance(self.step_recipe, Unset):
            step_recipe = UNSET
        else:
            step_recipe = self.step_recipe

        step_recipe_data = self.step_recipe_data

        numrecipe = self.numrecipe

        show_ingredients_table = self.show_ingredients_table

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if id is not UNSET:
            field_dict["id"] = id
        if name is not UNSET:
            field_dict["name"] = name
        if instruction is not UNSET:
            field_dict["instruction"] = instruction
        if ingredients is not UNSET:
            field_dict["ingredients"] = ingredients
        if instructions_markdown is not UNSET:
            field_dict["instructions_markdown"] = instructions_markdown
        if time is not UNSET:
            field_dict["time"] = time
        if order is not UNSET:
            field_dict["order"] = order
        if show_as_header is not UNSET:
            field_dict["show_as_header"] = show_as_header
        if file is not UNSET:
            field_dict["file"] = file
        if step_recipe is not UNSET:
            field_dict["step_recipe"] = step_recipe
        if step_recipe_data is not UNSET:
            field_dict["step_recipe_data"] = step_recipe_data
        if numrecipe is not UNSET:
            field_dict["numrecipe"] = numrecipe
        if show_ingredients_table is not UNSET:
            field_dict["show_ingredients_table"] = show_ingredients_table

        return field_dict

    def to_multipart(self) -> types.RequestFiles:
        files: types.RequestFiles = []

        if not isinstance(self.id, Unset):
            files.append(("id", (None, str(self.id).encode(), "text/plain")))

        if not isinstance(self.name, Unset):
            files.append(("name", (None, str(self.name).encode(), "text/plain")))

        if not isinstance(self.instruction, Unset):
            files.append(("instruction", (None, str(self.instruction).encode(), "text/plain")))

        if not isinstance(self.ingredients, Unset):
            for ingredients_item_element in self.ingredients:
                files.append(
                    ("ingredients", (None, json.dumps(ingredients_item_element.to_dict()).encode(), "application/json"))
                )

        if not isinstance(self.instructions_markdown, Unset):
            files.append(("instructions_markdown", (None, str(self.instructions_markdown).encode(), "text/plain")))

        if not isinstance(self.time, Unset):
            files.append(("time", (None, str(self.time).encode(), "text/plain")))

        if not isinstance(self.order, Unset):
            files.append(("order", (None, str(self.order).encode(), "text/plain")))

        if not isinstance(self.show_as_header, Unset):
            files.append(("show_as_header", (None, str(self.show_as_header).encode(), "text/plain")))

        if not isinstance(self.file, Unset):
            if self.file is None:
                files.append(("file", (None, str(self.file).encode(), "text/plain")))
            else:
                files.append(("file", (None, json.dumps(self.file.to_dict()).encode(), "application/json")))

        if not isinstance(self.step_recipe, Unset):
            if isinstance(self.step_recipe, int):
                files.append(("step_recipe", (None, str(self.step_recipe).encode(), "text/plain")))
            else:
                files.append(("step_recipe", (None, str(self.step_recipe).encode(), "text/plain")))

        if not isinstance(self.step_recipe_data, Unset):
            files.append(("step_recipe_data", (None, str(self.step_recipe_data).encode(), "text/plain")))

        if not isinstance(self.numrecipe, Unset):
            files.append(("numrecipe", (None, str(self.numrecipe).encode(), "text/plain")))

        if not isinstance(self.show_ingredients_table, Unset):
            files.append(("show_ingredients_table", (None, str(self.show_ingredients_table).encode(), "text/plain")))

        for prop_name, prop in self.additional_properties.items():
            files.append((prop_name, (None, str(prop).encode(), "text/plain")))

        return files

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.ingredient import Ingredient
        from ..models.user_file_view import UserFileView

        d = dict(src_dict)
        id = d.pop("id", UNSET)

        name = d.pop("name", UNSET)

        instruction = d.pop("instruction", UNSET)

        _ingredients = d.pop("ingredients", UNSET)
        ingredients: list[Ingredient] | Unset = UNSET
        if _ingredients is not UNSET:
            ingredients = []
            for ingredients_item_data in _ingredients:
                ingredients_item = Ingredient.from_dict(ingredients_item_data)

                ingredients.append(ingredients_item)

        instructions_markdown = d.pop("instructions_markdown", UNSET)

        time = d.pop("time", UNSET)

        order = d.pop("order", UNSET)

        show_as_header = d.pop("show_as_header", UNSET)

        def _parse_file(data: object) -> None | Unset | UserFileView:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                file_type_1 = UserFileView.from_dict(data)

                return file_type_1
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(None | Unset | UserFileView, data)

        file = _parse_file(d.pop("file", UNSET))

        def _parse_step_recipe(data: object) -> int | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(int | None | Unset, data)

        step_recipe = _parse_step_recipe(d.pop("step_recipe", UNSET))

        step_recipe_data = d.pop("step_recipe_data", UNSET)

        numrecipe = d.pop("numrecipe", UNSET)

        show_ingredients_table = d.pop("show_ingredients_table", UNSET)

        patched_step = cls(
            id=id,
            name=name,
            instruction=instruction,
            ingredients=ingredients,
            instructions_markdown=instructions_markdown,
            time=time,
            order=order,
            show_as_header=show_as_header,
            file=file,
            step_recipe=step_recipe,
            step_recipe_data=step_recipe_data,
            numrecipe=numrecipe,
            show_ingredients_table=show_ingredients_table,
        )

        patched_step.additional_properties = d
        return patched_step

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

import datetime
import json
from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, cast
from uuid import UUID

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from .. import types
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.group import Group


T = TypeVar("T", bound="InviteLink")


@_attrs_define
class InviteLink:
    """Adds nested create feature

    Attributes:
        uuid (UUID):
        group (Group): Moves `UniqueValidator`'s from the validation stage to the save stage.
            It solves the problem with nested validation for unique fields on update.

            If you want more details, you can read related issues and articles:
            https://github.com/beda-software/drf-writable-nested/issues/1
            http://www.django-rest-framework.org/api-guide/validators/#updating-nested-serializers

            Example of usage:
            ```
                class Child(models.Model):
                field = models.CharField(unique=True)


            class Parent(models.Model):
                child = models.ForeignKey('Child')


            class ChildSerializer(UniqueFieldsMixin, serializers.ModelSerializer):
                class Meta:
                    model = Child


            class ParentSerializer(NestedUpdateMixin, serializers.ModelSerializer):
                child = ChildSerializer()

                class Meta:
                    model = Parent
            ```

            Note: `UniqueFieldsMixin` must be applied only on the serializer
            which has unique fields.

            Note: When you are using both mixins
            (`UniqueFieldsMixin` and `NestedCreateMixin` or `NestedUpdateMixin`)
            you should put `UniqueFieldsMixin` ahead.
        used_by (int | None):
        created_by (int):
        created_at (datetime.datetime):
        id (int | Unset):
        email (str | Unset):
        valid_until (datetime.date | Unset):
        reusable (bool | Unset):
        internal_note (None | str | Unset):
    """

    uuid: UUID
    group: Group
    used_by: int | None
    created_by: int
    created_at: datetime.datetime
    id: int | Unset = UNSET
    email: str | Unset = UNSET
    valid_until: datetime.date | Unset = UNSET
    reusable: bool | Unset = UNSET
    internal_note: None | str | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        uuid = str(self.uuid)

        group = self.group.to_dict()

        used_by: int | None
        used_by = self.used_by

        created_by = self.created_by

        created_at = self.created_at.isoformat()

        id = self.id

        email = self.email

        valid_until: str | Unset = UNSET
        if not isinstance(self.valid_until, Unset):
            valid_until = self.valid_until.isoformat()

        reusable = self.reusable

        internal_note: None | str | Unset
        if isinstance(self.internal_note, Unset):
            internal_note = UNSET
        else:
            internal_note = self.internal_note

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "uuid": uuid,
                "group": group,
                "used_by": used_by,
                "created_by": created_by,
                "created_at": created_at,
            }
        )
        if id is not UNSET:
            field_dict["id"] = id
        if email is not UNSET:
            field_dict["email"] = email
        if valid_until is not UNSET:
            field_dict["valid_until"] = valid_until
        if reusable is not UNSET:
            field_dict["reusable"] = reusable
        if internal_note is not UNSET:
            field_dict["internal_note"] = internal_note

        return field_dict

    def to_multipart(self) -> types.RequestFiles:
        files: types.RequestFiles = []

        files.append(("uuid", (None, str(self.uuid), "text/plain")))

        files.append(("group", (None, json.dumps(self.group.to_dict()).encode(), "application/json")))

        if isinstance(self.used_by, int):
            files.append(("used_by", (None, str(self.used_by).encode(), "text/plain")))
        else:
            files.append(("used_by", (None, str(self.used_by).encode(), "text/plain")))

        files.append(("created_by", (None, str(self.created_by).encode(), "text/plain")))

        files.append(("created_at", (None, self.created_at.isoformat().encode(), "text/plain")))

        if not isinstance(self.id, Unset):
            files.append(("id", (None, str(self.id).encode(), "text/plain")))

        if not isinstance(self.email, Unset):
            files.append(("email", (None, str(self.email).encode(), "text/plain")))

        if not isinstance(self.valid_until, Unset):
            files.append(("valid_until", (None, self.valid_until.isoformat().encode(), "text/plain")))

        if not isinstance(self.reusable, Unset):
            files.append(("reusable", (None, str(self.reusable).encode(), "text/plain")))

        if not isinstance(self.internal_note, Unset):
            if isinstance(self.internal_note, str):
                files.append(("internal_note", (None, str(self.internal_note).encode(), "text/plain")))
            else:
                files.append(("internal_note", (None, str(self.internal_note).encode(), "text/plain")))

        for prop_name, prop in self.additional_properties.items():
            files.append((prop_name, (None, str(prop).encode(), "text/plain")))

        return files

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.group import Group

        d = dict(src_dict)
        uuid = UUID(d.pop("uuid"))

        group = Group.from_dict(d.pop("group"))

        def _parse_used_by(data: object) -> int | None:
            if data is None:
                return data
            return cast(int | None, data)

        used_by = _parse_used_by(d.pop("used_by"))

        created_by = d.pop("created_by")

        created_at = isoparse(d.pop("created_at"))

        id = d.pop("id", UNSET)

        email = d.pop("email", UNSET)

        _valid_until = d.pop("valid_until", UNSET)
        valid_until: datetime.date | Unset
        if isinstance(_valid_until, Unset):
            valid_until = UNSET
        else:
            valid_until = isoparse(_valid_until).date()

        reusable = d.pop("reusable", UNSET)

        def _parse_internal_note(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        internal_note = _parse_internal_note(d.pop("internal_note", UNSET))

        invite_link = cls(
            uuid=uuid,
            group=group,
            used_by=used_by,
            created_by=created_by,
            created_at=created_at,
            id=id,
            email=email,
            valid_until=valid_until,
            reusable=reusable,
            internal_note=internal_note,
        )

        invite_link.additional_properties = d
        return invite_link

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.default_page_enum import DefaultPageEnum
from ..models.theme_enum import ThemeEnum
from ..models.user_preference_nav_text_color_enum import UserPreferenceNavTextColorEnum
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.food_inherit_field import FoodInheritField
    from ..models.user import User
    from ..models.user_file_view import UserFileView


T = TypeVar("T", bound="UserPreference")


@_attrs_define
class UserPreference:
    """Adds nested create feature

    Attributes:
        user (User): Adds nested create feature
        food_inherit_default (FoodInheritField): Moves `UniqueValidator`'s from the validation stage to the save stage.
            It solves the problem with nested validation for unique fields on update.

            If you want more details, you can read related issues and articles:
            https://github.com/beda-software/drf-writable-nested/issues/1
            http://www.django-rest-framework.org/api-guide/validators/#updating-nested-serializers

            Example of usage:
            ```
                class Child(models.Model):
                field = models.CharField(unique=True)


            class Parent(models.Model):
                child = models.ForeignKey('Child')


            class ChildSerializer(UniqueFieldsMixin, serializers.ModelSerializer):
                class Meta:
                    model = Child


            class ParentSerializer(NestedUpdateMixin, serializers.ModelSerializer):
                child = ChildSerializer()

                class Meta:
                    model = Parent
            ```

            Note: `UniqueFieldsMixin` must be applied only on the serializer
            which has unique fields.

            Note: When you are using both mixins
            (`UniqueFieldsMixin` and `NestedCreateMixin` or `NestedUpdateMixin`)
            you should put `UniqueFieldsMixin` ahead.
        food_children_exist (bool):
        image (None | Unset | UserFileView):
        theme (ThemeEnum | Unset): * `TANDOOR` - Tandoor
            * `BOOTSTRAP` - Bootstrap
            * `DARKLY` - Darkly
            * `FLATLY` - Flatly
            * `SUPERHERO` - Superhero
            * `TANDOOR_DARK` - Tandoor Dark (INCOMPLETE)
        nav_bg_color (str | Unset):
        nav_text_color (UserPreferenceNavTextColorEnum | Unset): * `LIGHT` - Light
            * `DARK` - Dark
        nav_show_logo (bool | Unset):
        default_unit (str | Unset):
        default_page (DefaultPageEnum | Unset): * `SEARCH` - Search
            * `PLAN` - Meal-Plan
            * `BOOKS` - Books
            * `SHOPPING` - Shopping
        use_fractions (bool | Unset):
        use_kj (bool | Unset):
        plan_share (list[User] | None | Unset):
        nav_sticky (bool | Unset):
        ingredient_decimals (int | Unset):
        comments (bool | Unset):
        shopping_auto_sync (int | Unset):
        mealplan_autoadd_shopping (bool | Unset):
        default_delay (float | Unset):
        mealplan_autoinclude_related (bool | Unset):
        mealplan_autoexclude_onhand (bool | Unset):
        shopping_share (list[User] | None | Unset):
        shopping_recent_days (int | Unset):
        csv_delim (str | Unset):
        csv_prefix (str | Unset):
        shopping_update_food_lists (bool | Unset):
        filter_to_supermarket (bool | Unset):
        shopping_add_onhand (bool | Unset):
        left_handed (bool | Unset):
        show_step_ingredients (bool | Unset):
    """

    user: User
    food_inherit_default: FoodInheritField
    food_children_exist: bool
    image: None | Unset | UserFileView = UNSET
    theme: ThemeEnum | Unset = UNSET
    nav_bg_color: str | Unset = UNSET
    nav_text_color: UserPreferenceNavTextColorEnum | Unset = UNSET
    nav_show_logo: bool | Unset = UNSET
    default_unit: str | Unset = UNSET
    default_page: DefaultPageEnum | Unset = UNSET
    use_fractions: bool | Unset = UNSET
    use_kj: bool | Unset = UNSET
    plan_share: list[User] | None | Unset = UNSET
    nav_sticky: bool | Unset = UNSET
    ingredient_decimals: int | Unset = UNSET
    comments: bool | Unset = UNSET
    shopping_auto_sync: int | Unset = UNSET
    mealplan_autoadd_shopping: bool | Unset = UNSET
    default_delay: float | Unset = UNSET
    mealplan_autoinclude_related: bool | Unset = UNSET
    mealplan_autoexclude_onhand: bool | Unset = UNSET
    shopping_share: list[User] | None | Unset = UNSET
    shopping_recent_days: int | Unset = UNSET
    csv_delim: str | Unset = UNSET
    csv_prefix: str | Unset = UNSET
    shopping_update_food_lists: bool | Unset = UNSET
    filter_to_supermarket: bool | Unset = UNSET
    shopping_add_onhand: bool | Unset = UNSET
    left_handed: bool | Unset = UNSET
    show_step_ingredients: bool | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        from ..models.user_file_view import UserFileView

        user = self.user.to_dict()

        food_inherit_default = self.food_inherit_default.to_dict()

        food_children_exist = self.food_children_exist

        image: dict[str, Any] | None | Unset
        if isinstance(self.image, Unset):
            image = UNSET
        elif isinstance(self.image, UserFileView):
            image = self.image.to_dict()
        else:
            image = self.image

        theme: str | Unset = UNSET
        if not isinstance(self.theme, Unset):
            theme = self.theme.value

        nav_bg_color = self.nav_bg_color

        nav_text_color: str | Unset = UNSET
        if not isinstance(self.nav_text_color, Unset):
            nav_text_color = self.nav_text_color.value

        nav_show_logo = self.nav_show_logo

        default_unit = self.default_unit

        default_page: str | Unset = UNSET
        if not isinstance(self.default_page, Unset):
            default_page = self.default_page.value

        use_fractions = self.use_fractions

        use_kj = self.use_kj

        plan_share: list[dict[str, Any]] | None | Unset
        if isinstance(self.plan_share, Unset):
            plan_share = UNSET
        elif isinstance(self.plan_share, list):
            plan_share = []
            for plan_share_type_0_item_data in self.plan_share:
                plan_share_type_0_item = plan_share_type_0_item_data.to_dict()
                plan_share.append(plan_share_type_0_item)

        else:
            plan_share = self.plan_share

        nav_sticky = self.nav_sticky

        ingredient_decimals = self.ingredient_decimals

        comments = self.comments

        shopping_auto_sync = self.shopping_auto_sync

        mealplan_autoadd_shopping = self.mealplan_autoadd_shopping

        default_delay = self.default_delay

        mealplan_autoinclude_related = self.mealplan_autoinclude_related

        mealplan_autoexclude_onhand = self.mealplan_autoexclude_onhand

        shopping_share: list[dict[str, Any]] | None | Unset
        if isinstance(self.shopping_share, Unset):
            shopping_share = UNSET
        elif isinstance(self.shopping_share, list):
            shopping_share = []
            for shopping_share_type_0_item_data in self.shopping_share:
                shopping_share_type_0_item = shopping_share_type_0_item_data.to_dict()
                shopping_share.append(shopping_share_type_0_item)

        else:
            shopping_share = self.shopping_share

        shopping_recent_days = self.shopping_recent_days

        csv_delim = self.csv_delim

        csv_prefix = self.csv_prefix

        shopping_update_food_lists = self.shopping_update_food_lists

        filter_to_supermarket = self.filter_to_supermarket

        shopping_add_onhand = self.shopping_add_onhand

        left_handed = self.left_handed

        show_step_ingredients = self.show_step_ingredients

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "user": user,
                "food_inherit_default": food_inherit_default,
                "food_children_exist": food_children_exist,
            }
        )
        if image is not UNSET:
            field_dict["image"] = image
        if theme is not UNSET:
            field_dict["theme"] = theme
        if nav_bg_color is not UNSET:
            field_dict["nav_bg_color"] = nav_bg_color
        if nav_text_color is not UNSET:
            field_dict["nav_text_color"] = nav_text_color
        if nav_show_logo is not UNSET:
            field_dict["nav_show_logo"] = nav_show_logo
        if default_unit is not UNSET:
            field_dict["default_unit"] = default_unit
        if default_page is not UNSET:
            field_dict["default_page"] = default_page
        if use_fractions is not UNSET:
            field_dict["use_fractions"] = use_fractions
        if use_kj is not UNSET:
            field_dict["use_kj"] = use_kj
        if plan_share is not UNSET:
            field_dict["plan_share"] = plan_share
        if nav_sticky is not UNSET:
            field_dict["nav_sticky"] = nav_sticky
        if ingredient_decimals is not UNSET:
            field_dict["ingredient_decimals"] = ingredient_decimals
        if comments is not UNSET:
            field_dict["comments"] = comments
        if shopping_auto_sync is not UNSET:
            field_dict["shopping_auto_sync"] = shopping_auto_sync
        if mealplan_autoadd_shopping is not UNSET:
            field_dict["mealplan_autoadd_shopping"] = mealplan_autoadd_shopping
        if default_delay is not UNSET:
            field_dict["default_delay"] = default_delay
        if mealplan_autoinclude_related is not UNSET:
            field_dict["mealplan_autoinclude_related"] = mealplan_autoinclude_related
        if mealplan_autoexclude_onhand is not UNSET:
            field_dict["mealplan_autoexclude_onhand"] = mealplan_autoexclude_onhand
        if shopping_share is not UNSET:
            field_dict["shopping_share"] = shopping_share
        if shopping_recent_days is not UNSET:
            field_dict["shopping_recent_days"] = shopping_recent_days
        if csv_delim is not UNSET:
            field_dict["csv_delim"] = csv_delim
        if csv_prefix is not UNSET:
            field_dict["csv_prefix"] = csv_prefix
        if shopping_update_food_lists is not UNSET:
            field_dict["shopping_update_food_lists"] = shopping_update_food_lists
        if filter_to_supermarket is not UNSET:
            field_dict["filter_to_supermarket"] = filter_to_supermarket
        if shopping_add_onhand is not UNSET:
            field_dict["shopping_add_onhand"] = shopping_add_onhand
        if left_handed is not UNSET:
            field_dict["left_handed"] = left_handed
        if show_step_ingredients is not UNSET:
            field_dict["show_step_ingredients"] = show_step_ingredients

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.food_inherit_field import FoodInheritField
        from ..models.user import User
        from ..models.user_file_view import UserFileView

        d = dict(src_dict)
        user = User.from_dict(d.pop("user"))

        food_inherit_default = FoodInheritField.from_dict(d.pop("food_inherit_default"))

        food_children_exist = d.pop("food_children_exist")

        def _parse_image(data: object) -> None | Unset | UserFileView:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                image_type_1 = UserFileView.from_dict(data)

                return image_type_1
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(None | Unset | UserFileView, data)

        image = _parse_image(d.pop("image", UNSET))

        _theme = d.pop("theme", UNSET)
        theme: ThemeEnum | Unset
        if isinstance(_theme, Unset):
            theme = UNSET
        else:
            theme = ThemeEnum(_theme)

        nav_bg_color = d.pop("nav_bg_color", UNSET)

        _nav_text_color = d.pop("nav_text_color", UNSET)
        nav_text_color: UserPreferenceNavTextColorEnum | Unset
        if isinstance(_nav_text_color, Unset):
            nav_text_color = UNSET
        else:
            nav_text_color = UserPreferenceNavTextColorEnum(_nav_text_color)

        nav_show_logo = d.pop("nav_show_logo", UNSET)

        default_unit = d.pop("default_unit", UNSET)

        _default_page = d.pop("default_page", UNSET)
        default_page: DefaultPageEnum | Unset
        if isinstance(_default_page, Unset):
            default_page = UNSET
        else:
            default_page = DefaultPageEnum(_default_page)

        use_fractions = d.pop("use_fractions", UNSET)

        use_kj = d.pop("use_kj", UNSET)

        def _parse_plan_share(data: object) -> list[User] | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                plan_share_type_0 = []
                _plan_share_type_0 = data
                for plan_share_type_0_item_data in _plan_share_type_0:
                    plan_share_type_0_item = User.from_dict(plan_share_type_0_item_data)

                    plan_share_type_0.append(plan_share_type_0_item)

                return plan_share_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(list[User] | None | Unset, data)

        plan_share = _parse_plan_share(d.pop("plan_share", UNSET))

        nav_sticky = d.pop("nav_sticky", UNSET)

        ingredient_decimals = d.pop("ingredient_decimals", UNSET)

        comments = d.pop("comments", UNSET)

        shopping_auto_sync = d.pop("shopping_auto_sync", UNSET)

        mealplan_autoadd_shopping = d.pop("mealplan_autoadd_shopping", UNSET)

        default_delay = d.pop("default_delay", UNSET)

        mealplan_autoinclude_related = d.pop("mealplan_autoinclude_related", UNSET)

        mealplan_autoexclude_onhand = d.pop("mealplan_autoexclude_onhand", UNSET)

        def _parse_shopping_share(data: object) -> list[User] | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                shopping_share_type_0 = []
                _shopping_share_type_0 = data
                for shopping_share_type_0_item_data in _shopping_share_type_0:
                    shopping_share_type_0_item = User.from_dict(shopping_share_type_0_item_data)

                    shopping_share_type_0.append(shopping_share_type_0_item)

                return shopping_share_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(list[User] | None | Unset, data)

        shopping_share = _parse_shopping_share(d.pop("shopping_share", UNSET))

        shopping_recent_days = d.pop("shopping_recent_days", UNSET)

        csv_delim = d.pop("csv_delim", UNSET)

        csv_prefix = d.pop("csv_prefix", UNSET)

        shopping_update_food_lists = d.pop("shopping_update_food_lists", UNSET)

        filter_to_supermarket = d.pop("filter_to_supermarket", UNSET)

        shopping_add_onhand = d.pop("shopping_add_onhand", UNSET)

        left_handed = d.pop("left_handed", UNSET)

        show_step_ingredients = d.pop("show_step_ingredients", UNSET)

        user_preference = cls(
            user=user,
            food_inherit_default=food_inherit_default,
            food_children_exist=food_children_exist,
            image=image,
            theme=theme,
            nav_bg_color=nav_bg_color,
            nav_text_color=nav_text_color,
            nav_show_logo=nav_show_logo,
            default_unit=default_unit,
            default_page=default_page,
            use_fractions=use_fractions,
            use_kj=use_kj,
            plan_share=plan_share,
            nav_sticky=nav_sticky,
            ingredient_decimals=ingredient_decimals,
            comments=comments,
            shopping_auto_sync=shopping_auto_sync,
            mealplan_autoadd_shopping=mealplan_autoadd_shopping,
            default_delay=default_delay,
            mealplan_autoinclude_related=mealplan_autoinclude_related,
            mealplan_autoexclude_onhand=mealplan_autoexclude_onhand,
            shopping_share=shopping_share,
            shopping_recent_days=shopping_recent_days,
            csv_delim=csv_delim,
            csv_prefix=csv_prefix,
            shopping_update_food_lists=shopping_update_food_lists,
            filter_to_supermarket=filter_to_supermarket,
            shopping_add_onhand=shopping_add_onhand,
            left_handed=left_handed,
            show_step_ingredients=show_step_ingredients,
        )

        user_preference.additional_properties = d
        return user_preference

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

import datetime
import json
from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from .. import types
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.user import User


T = TypeVar("T", bound="AutoMealPlan")


@_attrs_define
class AutoMealPlan:
    """
    Attributes:
        start_date (datetime.datetime):
        end_date (datetime.datetime):
        meal_type_id (int):
        keyword_ids (list[Any]):
        servings (float):
        addshopping (bool):
        shared (list[User] | None | Unset):
    """

    start_date: datetime.datetime
    end_date: datetime.datetime
    meal_type_id: int
    keyword_ids: list[Any]
    servings: float
    addshopping: bool
    shared: list[User] | None | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        start_date = self.start_date.isoformat()

        end_date = self.end_date.isoformat()

        meal_type_id = self.meal_type_id

        keyword_ids = self.keyword_ids

        servings = self.servings

        addshopping = self.addshopping

        shared: list[dict[str, Any]] | None | Unset
        if isinstance(self.shared, Unset):
            shared = UNSET
        elif isinstance(self.shared, list):
            shared = []
            for shared_type_0_item_data in self.shared:
                shared_type_0_item = shared_type_0_item_data.to_dict()
                shared.append(shared_type_0_item)

        else:
            shared = self.shared

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "start_date": start_date,
                "end_date": end_date,
                "meal_type_id": meal_type_id,
                "keyword_ids": keyword_ids,
                "servings": servings,
                "addshopping": addshopping,
            }
        )
        if shared is not UNSET:
            field_dict["shared"] = shared

        return field_dict

    def to_multipart(self) -> types.RequestFiles:
        files: types.RequestFiles = []

        files.append(("start_date", (None, self.start_date.isoformat().encode(), "text/plain")))

        files.append(("end_date", (None, self.end_date.isoformat().encode(), "text/plain")))

        files.append(("meal_type_id", (None, str(self.meal_type_id).encode(), "text/plain")))

        for keyword_ids_item_element in self.keyword_ids:
            files.append(("keyword_ids", (None, str(keyword_ids_item_element).encode(), "text/plain")))

        files.append(("servings", (None, str(self.servings).encode(), "text/plain")))

        files.append(("addshopping", (None, str(self.addshopping).encode(), "text/plain")))

        if not isinstance(self.shared, Unset):
            if isinstance(self.shared, list):
                for shared_type_0_item_element in self.shared:
                    files.append(
                        (
                            "shared",
                            (None, json.dumps(shared_type_0_item_element.to_dict()).encode(), "application/json"),
                        )
                    )
            else:
                files.append(("shared", (None, str(self.shared).encode(), "text/plain")))

        for prop_name, prop in self.additional_properties.items():
            files.append((prop_name, (None, str(prop).encode(), "text/plain")))

        return files

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.user import User

        d = dict(src_dict)
        start_date = isoparse(d.pop("start_date"))

        end_date = isoparse(d.pop("end_date"))

        meal_type_id = d.pop("meal_type_id")

        keyword_ids = cast(list[Any], d.pop("keyword_ids"))

        servings = d.pop("servings")

        addshopping = d.pop("addshopping")

        def _parse_shared(data: object) -> list[User] | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                shared_type_0 = []
                _shared_type_0 = data
                for shared_type_0_item_data in _shared_type_0:
                    shared_type_0_item = User.from_dict(shared_type_0_item_data)

                    shared_type_0.append(shared_type_0_item)

                return shared_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(list[User] | None | Unset, data)

        shared = _parse_shared(d.pop("shared", UNSET))

        auto_meal_plan = cls(
            start_date=start_date,
            end_date=end_date,
            meal_type_id=meal_type_id,
            keyword_ids=keyword_ids,
            servings=servings,
            addshopping=addshopping,
            shared=shared,
        )

        auto_meal_plan.additional_properties = d
        return auto_meal_plan

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

import json
from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from .. import types
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.user import User


T = TypeVar("T", bound="CustomFilter")


@_attrs_define
class CustomFilter:
    """Adds nested create feature

    Attributes:
        name (str):
        search (str):
        created_by (int):
        id (int | Unset):
        shared (list[User] | Unset):
    """

    name: str
    search: str
    created_by: int
    id: int | Unset = UNSET
    shared: list[User] | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        name = self.name

        search = self.search

        created_by = self.created_by

        id = self.id

        shared: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.shared, Unset):
            shared = []
            for shared_item_data in self.shared:
                shared_item = shared_item_data.to_dict()
                shared.append(shared_item)

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "name": name,
                "search": search,
                "created_by": created_by,
            }
        )
        if id is not UNSET:
            field_dict["id"] = id
        if shared is not UNSET:
            field_dict["shared"] = shared

        return field_dict

    def to_multipart(self) -> types.RequestFiles:
        files: types.RequestFiles = []

        files.append(("name", (None, str(self.name).encode(), "text/plain")))

        files.append(("search", (None, str(self.search).encode(), "text/plain")))

        files.append(("created_by", (None, str(self.created_by).encode(), "text/plain")))

        if not isinstance(self.id, Unset):
            files.append(("id", (None, str(self.id).encode(), "text/plain")))

        if not isinstance(self.shared, Unset):
            for shared_item_element in self.shared:
                files.append(("shared", (None, json.dumps(shared_item_element.to_dict()).encode(), "application/json")))

        for prop_name, prop in self.additional_properties.items():
            files.append((prop_name, (None, str(prop).encode(), "text/plain")))

        return files

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.user import User

        d = dict(src_dict)
        name = d.pop("name")

        search = d.pop("search")

        created_by = d.pop("created_by")

        id = d.pop("id", UNSET)

        _shared = d.pop("shared", UNSET)
        shared: list[User] | Unset = UNSET
        if _shared is not UNSET:
            shared = []
            for shared_item_data in _shared:
                shared_item = User.from_dict(shared_item_data)

                shared.append(shared_item)

        custom_filter = cls(
            name=name,
            search=search,
            created_by=created_by,
            id=id,
            shared=shared,
        )

        custom_filter.additional_properties = d
        return custom_filter

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.source_import_ingredient import SourceImportIngredient


T = TypeVar("T", bound="SourceImportStep")


@_attrs_define
class SourceImportStep:
    """
    Attributes:
        instruction (str):
        ingredients (list[SourceImportIngredient]):
        show_ingredients_table (bool | Unset):  Default: True.
    """

    instruction: str
    ingredients: list[SourceImportIngredient]
    show_ingredients_table: bool | Unset = True
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        instruction = self.instruction

        ingredients = []
        for ingredients_item_data in self.ingredients:
            ingredients_item = ingredients_item_data.to_dict()
            ingredients.append(ingredients_item)

        show_ingredients_table = self.show_ingredients_table

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "instruction": instruction,
                "ingredients": ingredients,
            }
        )
        if show_ingredients_table is not UNSET:
            field_dict["show_ingredients_table"] = show_ingredients_table

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.source_import_ingredient import SourceImportIngredient

        d = dict(src_dict)
        instruction = d.pop("instruction")

        ingredients = []
        _ingredients = d.pop("ingredients")
        for ingredients_item_data in _ingredients:
            ingredients_item = SourceImportIngredient.from_dict(ingredients_item_data)

            ingredients.append(ingredients_item)

        show_ingredients_table = d.pop("show_ingredients_table", UNSET)

        source_import_step = cls(
            instruction=instruction,
            ingredients=ingredients,
            show_ingredients_table=show_ingredients_table,
        )

        source_import_step.additional_properties = d
        return source_import_step

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="Group")


@_attrs_define
class Group:
    """Moves `UniqueValidator`'s from the validation stage to the save stage.
    It solves the problem with nested validation for unique fields on update.

    If you want more details, you can read related issues and articles:
    https://github.com/beda-software/drf-writable-nested/issues/1
    http://www.django-rest-framework.org/api-guide/validators/#updating-nested-serializers

    Example of usage:
    ```
        class Child(models.Model):
        field = models.CharField(unique=True)


    class Parent(models.Model):
        child = models.ForeignKey('Child')


    class ChildSerializer(UniqueFieldsMixin, serializers.ModelSerializer):
        class Meta:
            model = Child


    class ParentSerializer(NestedUpdateMixin, serializers.ModelSerializer):
        child = ChildSerializer()

        class Meta:
            model = Parent
    ```

    Note: `UniqueFieldsMixin` must be applied only on the serializer
    which has unique fields.

    Note: When you are using both mixins
    (`UniqueFieldsMixin` and `NestedCreateMixin` or `NestedUpdateMixin`)
    you should put `UniqueFieldsMixin` ahead.

        Attributes:
            name (str):
            id (int | Unset):
    """

    name: str
    id: int | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        name = self.name

        id = self.id

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "name": name,
            }
        )
        if id is not UNSET:
            field_dict["id"] = id

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        name = d.pop("name")

        id = d.pop("id", UNSET)

        group = cls(
            name=name,
            id=id,
        )

        group.additional_properties = d
        return group

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

import datetime
from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from .. import types
from ..types import UNSET, Unset

T = TypeVar("T", bound="AccessToken")


@_attrs_define
class AccessToken:
    """
    Attributes:
        token (str):
        expires (datetime.datetime):
        created (datetime.datetime):
        updated (datetime.datetime):
        id (int | Unset):
        scope (str | Unset):
    """

    token: str
    expires: datetime.datetime
    created: datetime.datetime
    updated: datetime.datetime
    id: int | Unset = UNSET
    scope: str | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        token = self.token

        expires = self.expires.isoformat()

        created = self.created.isoformat()

        updated = self.updated.isoformat()

        id = self.id

        scope = self.scope

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "token": token,
                "expires": expires,
                "created": created,
                "updated": updated,
            }
        )
        if id is not UNSET:
            field_dict["id"] = id
        if scope is not UNSET:
            field_dict["scope"] = scope

        return field_dict

    def to_multipart(self) -> types.RequestFiles:
        files: types.RequestFiles = []

        files.append(("token", (None, str(self.token).encode(), "text/plain")))

        files.append(("expires", (None, self.expires.isoformat().encode(), "text/plain")))

        files.append(("created", (None, self.created.isoformat().encode(), "text/plain")))

        files.append(("updated", (None, self.updated.isoformat().encode(), "text/plain")))

        if not isinstance(self.id, Unset):
            files.append(("id", (None, str(self.id).encode(), "text/plain")))

        if not isinstance(self.scope, Unset):
            files.append(("scope", (None, str(self.scope).encode(), "text/plain")))

        for prop_name, prop in self.additional_properties.items():
            files.append((prop_name, (None, str(prop).encode(), "text/plain")))

        return files

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        token = d.pop("token")

        expires = isoparse(d.pop("expires"))

        created = isoparse(d.pop("created"))

        updated = isoparse(d.pop("updated"))

        id = d.pop("id", UNSET)

        scope = d.pop("scope", UNSET)

        access_token = cls(
            token=token,
            expires=expires,
            created=created,
            updated=updated,
            id=id,
            scope=scope,
        )

        access_token.additional_properties = d
        return access_token

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

if TYPE_CHECKING:
    from ..models.ingredient_simple import IngredientSimple


T = TypeVar("T", bound="IngredientParserResponse")


@_attrs_define
class IngredientParserResponse:
    """
    Attributes:
        ingredient (IngredientSimple | None):
        ingredients (list[IngredientSimple]):
    """

    ingredient: IngredientSimple | None
    ingredients: list[IngredientSimple]
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        from ..models.ingredient_simple import IngredientSimple

        ingredient: dict[str, Any] | None
        if isinstance(self.ingredient, IngredientSimple):
            ingredient = self.ingredient.to_dict()
        else:
            ingredient = self.ingredient

        ingredients = []
        for ingredients_item_data in self.ingredients:
            ingredients_item = ingredients_item_data.to_dict()
            ingredients.append(ingredients_item)

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "ingredient": ingredient,
                "ingredients": ingredients,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.ingredient_simple import IngredientSimple

        d = dict(src_dict)

        def _parse_ingredient(data: object) -> IngredientSimple | None:
            if data is None:
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                ingredient_type_1 = IngredientSimple.from_dict(data)

                return ingredient_type_1
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(IngredientSimple | None, data)

        ingredient = _parse_ingredient(d.pop("ingredient"))

        ingredients = []
        _ingredients = d.pop("ingredients")
        for ingredients_item_data in _ingredients:
            ingredients_item = IngredientSimple.from_dict(ingredients_item_data)

            ingredients.append(ingredients_item)

        ingredient_parser_response = cls(
            ingredient=ingredient,
            ingredients=ingredients,
        )

        ingredient_parser_response.additional_properties = d
        return ingredient_parser_response

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

import json
from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from .. import types
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.recipe_book import RecipeBook
    from ..models.recipe_overview import RecipeOverview


T = TypeVar("T", bound="RecipeBookEntry")


@_attrs_define
class RecipeBookEntry:
    """
    Attributes:
        book (int):
        book_content (RecipeBook): Adds nested create feature
        recipe (int):
        recipe_content (RecipeOverview): Adds nested create feature
        id (int | Unset):
    """

    book: int
    book_content: RecipeBook
    recipe: int
    recipe_content: RecipeOverview
    id: int | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        book = self.book

        book_content = self.book_content.to_dict()

        recipe = self.recipe

        recipe_content = self.recipe_content.to_dict()

        id = self.id

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "book": book,
                "book_content": book_content,
                "recipe": recipe,
                "recipe_content": recipe_content,
            }
        )
        if id is not UNSET:
            field_dict["id"] = id

        return field_dict

    def to_multipart(self) -> types.RequestFiles:
        files: types.RequestFiles = []

        files.append(("book", (None, str(self.book).encode(), "text/plain")))

        files.append(("book_content", (None, json.dumps(self.book_content.to_dict()).encode(), "application/json")))

        files.append(("recipe", (None, str(self.recipe).encode(), "text/plain")))

        files.append(("recipe_content", (None, json.dumps(self.recipe_content.to_dict()).encode(), "application/json")))

        if not isinstance(self.id, Unset):
            files.append(("id", (None, str(self.id).encode(), "text/plain")))

        for prop_name, prop in self.additional_properties.items():
            files.append((prop_name, (None, str(prop).encode(), "text/plain")))

        return files

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.recipe_book import RecipeBook
        from ..models.recipe_overview import RecipeOverview

        d = dict(src_dict)
        book = d.pop("book")

        book_content = RecipeBook.from_dict(d.pop("book_content"))

        recipe = d.pop("recipe")

        recipe_content = RecipeOverview.from_dict(d.pop("recipe_content"))

        id = d.pop("id", UNSET)

        recipe_book_entry = cls(
            book=book,
            book_content=book_content,
            recipe=recipe,
            recipe_content=recipe_content,
            id=id,
        )

        recipe_book_entry.additional_properties = d
        return recipe_book_entry

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

import datetime
from collections.abc import Mapping
from typing import Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from .. import types
from ..types import UNSET, Unset

T = TypeVar("T", bound="ShoppingListEntryBulk")


@_attrs_define
class ShoppingListEntryBulk:
    """
    Attributes:
        ids (list[Any]):
        timestamp (datetime.datetime):
        checked (bool | None | Unset):
        shopping_lists_add (list[int] | Unset):
        shopping_lists_remove (list[int] | Unset):
        shopping_lists_set (list[int] | Unset):
        shopping_lists_remove_all (bool | Unset):  Default: False.
    """

    ids: list[Any]
    timestamp: datetime.datetime
    checked: bool | None | Unset = UNSET
    shopping_lists_add: list[int] | Unset = UNSET
    shopping_lists_remove: list[int] | Unset = UNSET
    shopping_lists_set: list[int] | Unset = UNSET
    shopping_lists_remove_all: bool | Unset = False
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        ids = self.ids

        timestamp = self.timestamp.isoformat()

        checked: bool | None | Unset
        if isinstance(self.checked, Unset):
            checked = UNSET
        else:
            checked = self.checked

        shopping_lists_add: list[int] | Unset = UNSET
        if not isinstance(self.shopping_lists_add, Unset):
            shopping_lists_add = self.shopping_lists_add

        shopping_lists_remove: list[int] | Unset = UNSET
        if not isinstance(self.shopping_lists_remove, Unset):
            shopping_lists_remove = self.shopping_lists_remove

        shopping_lists_set: list[int] | Unset = UNSET
        if not isinstance(self.shopping_lists_set, Unset):
            shopping_lists_set = self.shopping_lists_set

        shopping_lists_remove_all = self.shopping_lists_remove_all

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "ids": ids,
                "timestamp": timestamp,
            }
        )
        if checked is not UNSET:
            field_dict["checked"] = checked
        if shopping_lists_add is not UNSET:
            field_dict["shopping_lists_add"] = shopping_lists_add
        if shopping_lists_remove is not UNSET:
            field_dict["shopping_lists_remove"] = shopping_lists_remove
        if shopping_lists_set is not UNSET:
            field_dict["shopping_lists_set"] = shopping_lists_set
        if shopping_lists_remove_all is not UNSET:
            field_dict["shopping_lists_remove_all"] = shopping_lists_remove_all

        return field_dict

    def to_multipart(self) -> types.RequestFiles:
        files: types.RequestFiles = []

        for ids_item_element in self.ids:
            files.append(("ids", (None, str(ids_item_element).encode(), "text/plain")))

        files.append(("timestamp", (None, self.timestamp.isoformat().encode(), "text/plain")))

        if not isinstance(self.checked, Unset):
            if isinstance(self.checked, bool):
                files.append(("checked", (None, str(self.checked).encode(), "text/plain")))
            else:
                files.append(("checked", (None, str(self.checked).encode(), "text/plain")))

        if not isinstance(self.shopping_lists_add, Unset):
            for shopping_lists_add_item_element in self.shopping_lists_add:
                files.append(
                    ("shopping_lists_add", (None, str(shopping_lists_add_item_element).encode(), "text/plain"))
                )

        if not isinstance(self.shopping_lists_remove, Unset):
            for shopping_lists_remove_item_element in self.shopping_lists_remove:
                files.append(
                    ("shopping_lists_remove", (None, str(shopping_lists_remove_item_element).encode(), "text/plain"))
                )

        if not isinstance(self.shopping_lists_set, Unset):
            for shopping_lists_set_item_element in self.shopping_lists_set:
                files.append(
                    ("shopping_lists_set", (None, str(shopping_lists_set_item_element).encode(), "text/plain"))
                )

        if not isinstance(self.shopping_lists_remove_all, Unset):
            files.append(
                ("shopping_lists_remove_all", (None, str(self.shopping_lists_remove_all).encode(), "text/plain"))
            )

        for prop_name, prop in self.additional_properties.items():
            files.append((prop_name, (None, str(prop).encode(), "text/plain")))

        return files

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        ids = cast(list[Any], d.pop("ids"))

        timestamp = isoparse(d.pop("timestamp"))

        def _parse_checked(data: object) -> bool | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(bool | None | Unset, data)

        checked = _parse_checked(d.pop("checked", UNSET))

        shopping_lists_add = cast(list[int], d.pop("shopping_lists_add", UNSET))

        shopping_lists_remove = cast(list[int], d.pop("shopping_lists_remove", UNSET))

        shopping_lists_set = cast(list[int], d.pop("shopping_lists_set", UNSET))

        shopping_lists_remove_all = d.pop("shopping_lists_remove_all", UNSET)

        shopping_list_entry_bulk = cls(
            ids=ids,
            timestamp=timestamp,
            checked=checked,
            shopping_lists_add=shopping_lists_add,
            shopping_lists_remove=shopping_lists_remove,
            shopping_lists_set=shopping_lists_set,
            shopping_lists_remove_all=shopping_lists_remove_all,
        )

        shopping_list_entry_bulk.additional_properties = d
        return shopping_list_entry_bulk

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from .. import types
from ..types import UNSET, Unset

T = TypeVar("T", bound="FoodBatchUpdate")


@_attrs_define
class FoodBatchUpdate:
    """
    Attributes:
        foods (list[int]):
        substitute_add (list[int]):
        substitute_remove (list[int]):
        substitute_set (list[int]):
        inherit_fields_add (list[int]):
        inherit_fields_remove (list[int]):
        inherit_fields_set (list[int]):
        child_inherit_fields_add (list[int]):
        child_inherit_fields_remove (list[int]):
        child_inherit_fields_set (list[int]):
        category (int | None | Unset):
        substitute_remove_all (bool | Unset):  Default: False.
        inherit_fields_remove_all (bool | Unset):  Default: False.
        child_inherit_fields_remove_all (bool | Unset):  Default: False.
        shopping_lists_add (list[int] | Unset):
        shopping_lists_remove (list[int] | Unset):
        shopping_lists_set (list[int] | Unset):
        shopping_lists_remove_all (bool | Unset):  Default: False.
        substitute_children (bool | None | Unset):
        substitute_siblings (bool | None | Unset):
        ignore_shopping (bool | None | Unset):
        on_hand (bool | None | Unset):
        parent_remove (bool | None | Unset):
        parent_set (int | None | Unset):
    """

    foods: list[int]
    substitute_add: list[int]
    substitute_remove: list[int]
    substitute_set: list[int]
    inherit_fields_add: list[int]
    inherit_fields_remove: list[int]
    inherit_fields_set: list[int]
    child_inherit_fields_add: list[int]
    child_inherit_fields_remove: list[int]
    child_inherit_fields_set: list[int]
    category: int | None | Unset = UNSET
    substitute_remove_all: bool | Unset = False
    inherit_fields_remove_all: bool | Unset = False
    child_inherit_fields_remove_all: bool | Unset = False
    shopping_lists_add: list[int] | Unset = UNSET
    shopping_lists_remove: list[int] | Unset = UNSET
    shopping_lists_set: list[int] | Unset = UNSET
    shopping_lists_remove_all: bool | Unset = False
    substitute_children: bool | None | Unset = UNSET
    substitute_siblings: bool | None | Unset = UNSET
    ignore_shopping: bool | None | Unset = UNSET
    on_hand: bool | None | Unset = UNSET
    parent_remove: bool | None | Unset = UNSET
    parent_set: int | None | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        foods = self.foods

        substitute_add = self.substitute_add

        substitute_remove = self.substitute_remove

        substitute_set = self.substitute_set

        inherit_fields_add = self.inherit_fields_add

        inherit_fields_remove = self.inherit_fields_remove

        inherit_fields_set = self.inherit_fields_set

        child_inherit_fields_add = self.child_inherit_fields_add

        child_inherit_fields_remove = self.child_inherit_fields_remove

        child_inherit_fields_set = self.child_inherit_fields_set

        category: int | None | Unset
        if isinstance(self.category, Unset):
            category = UNSET
        else:
            category = self.category

        substitute_remove_all = self.substitute_remove_all

        inherit_fields_remove_all = self.inherit_fields_remove_all

        child_inherit_fields_remove_all = self.child_inherit_fields_remove_all

        shopping_lists_add: list[int] | Unset = UNSET
        if not isinstance(self.shopping_lists_add, Unset):
            shopping_lists_add = self.shopping_lists_add

        shopping_lists_remove: list[int] | Unset = UNSET
        if not isinstance(self.shopping_lists_remove, Unset):
            shopping_lists_remove = self.shopping_lists_remove

        shopping_lists_set: list[int] | Unset = UNSET
        if not isinstance(self.shopping_lists_set, Unset):
            shopping_lists_set = self.shopping_lists_set

        shopping_lists_remove_all = self.shopping_lists_remove_all

        substitute_children: bool | None | Unset
        if isinstance(self.substitute_children, Unset):
            substitute_children = UNSET
        else:
            substitute_children = self.substitute_children

        substitute_siblings: bool | None | Unset
        if isinstance(self.substitute_siblings, Unset):
            substitute_siblings = UNSET
        else:
            substitute_siblings = self.substitute_siblings

        ignore_shopping: bool | None | Unset
        if isinstance(self.ignore_shopping, Unset):
            ignore_shopping = UNSET
        else:
            ignore_shopping = self.ignore_shopping

        on_hand: bool | None | Unset
        if isinstance(self.on_hand, Unset):
            on_hand = UNSET
        else:
            on_hand = self.on_hand

        parent_remove: bool | None | Unset
        if isinstance(self.parent_remove, Unset):
            parent_remove = UNSET
        else:
            parent_remove = self.parent_remove

        parent_set: int | None | Unset
        if isinstance(self.parent_set, Unset):
            parent_set = UNSET
        else:
            parent_set = self.parent_set

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "foods": foods,
                "substitute_add": substitute_add,
                "substitute_remove": substitute_remove,
                "substitute_set": substitute_set,
                "inherit_fields_add": inherit_fields_add,
                "inherit_fields_remove": inherit_fields_remove,
                "inherit_fields_set": inherit_fields_set,
                "child_inherit_fields_add": child_inherit_fields_add,
                "child_inherit_fields_remove": child_inherit_fields_remove,
                "child_inherit_fields_set": child_inherit_fields_set,
            }
        )
        if category is not UNSET:
            field_dict["category"] = category
        if substitute_remove_all is not UNSET:
            field_dict["substitute_remove_all"] = substitute_remove_all
        if inherit_fields_remove_all is not UNSET:
            field_dict["inherit_fields_remove_all"] = inherit_fields_remove_all
        if child_inherit_fields_remove_all is not UNSET:
            field_dict["child_inherit_fields_remove_all"] = child_inherit_fields_remove_all
        if shopping_lists_add is not UNSET:
            field_dict["shopping_lists_add"] = shopping_lists_add
        if shopping_lists_remove is not UNSET:
            field_dict["shopping_lists_remove"] = shopping_lists_remove
        if shopping_lists_set is not UNSET:
            field_dict["shopping_lists_set"] = shopping_lists_set
        if shopping_lists_remove_all is not UNSET:
            field_dict["shopping_lists_remove_all"] = shopping_lists_remove_all
        if substitute_children is not UNSET:
            field_dict["substitute_children"] = substitute_children
        if substitute_siblings is not UNSET:
            field_dict["substitute_siblings"] = substitute_siblings
        if ignore_shopping is not UNSET:
            field_dict["ignore_shopping"] = ignore_shopping
        if on_hand is not UNSET:
            field_dict["on_hand"] = on_hand
        if parent_remove is not UNSET:
            field_dict["parent_remove"] = parent_remove
        if parent_set is not UNSET:
            field_dict["parent_set"] = parent_set

        return field_dict

    def to_multipart(self) -> types.RequestFiles:
        files: types.RequestFiles = []

        for foods_item_element in self.foods:
            files.append(("foods", (None, str(foods_item_element).encode(), "text/plain")))

        for substitute_add_item_element in self.substitute_add:
            files.append(("substitute_add", (None, str(substitute_add_item_element).encode(), "text/plain")))

        for substitute_remove_item_element in self.substitute_remove:
            files.append(("substitute_remove", (None, str(substitute_remove_item_element).encode(), "text/plain")))

        for substitute_set_item_element in self.substitute_set:
            files.append(("substitute_set", (None, str(substitute_set_item_element).encode(), "text/plain")))

        for inherit_fields_add_item_element in self.inherit_fields_add:
            files.append(("inherit_fields_add", (None, str(inherit_fields_add_item_element).encode(), "text/plain")))

        for inherit_fields_remove_item_element in self.inherit_fields_remove:
            files.append(
                ("inherit_fields_remove", (None, str(inherit_fields_remove_item_element).encode(), "text/plain"))
            )

        for inherit_fields_set_item_element in self.inherit_fields_set:
            files.append(("inherit_fields_set", (None, str(inherit_fields_set_item_element).encode(), "text/plain")))

        for child_inherit_fields_add_item_element in self.child_inherit_fields_add:
            files.append(
                ("child_inherit_fields_add", (None, str(child_inherit_fields_add_item_element).encode(), "text/plain"))
            )

        for child_inherit_fields_remove_item_element in self.child_inherit_fields_remove:
            files.append(
                (
                    "child_inherit_fields_remove",
                    (None, str(child_inherit_fields_remove_item_element).encode(), "text/plain"),
                )
            )

        for child_inherit_fields_set_item_element in self.child_inherit_fields_set:
            files.append(
                ("child_inherit_fields_set", (None, str(child_inherit_fields_set_item_element).encode(), "text/plain"))
            )

        if not isinstance(self.category, Unset):
            if isinstance(self.category, int):
                files.append(("category", (None, str(self.category).encode(), "text/plain")))
            else:
                files.append(("category", (None, str(self.category).encode(), "text/plain")))

        if not isinstance(self.substitute_remove_all, Unset):
            files.append(("substitute_remove_all", (None, str(self.substitute_remove_all).encode(), "text/plain")))

        if not isinstance(self.inherit_fields_remove_all, Unset):
            files.append(
                ("inherit_fields_remove_all", (None, str(self.inherit_fields_remove_all).encode(), "text/plain"))
            )

        if not isinstance(self.child_inherit_fields_remove_all, Unset):
            files.append(
                (
                    "child_inherit_fields_remove_all",
                    (None, str(self.child_inherit_fields_remove_all).encode(), "text/plain"),
                )
            )

        if not isinstance(self.shopping_lists_add, Unset):
            for shopping_lists_add_item_element in self.shopping_lists_add:
                files.append(
                    ("shopping_lists_add", (None, str(shopping_lists_add_item_element).encode(), "text/plain"))
                )

        if not isinstance(self.shopping_lists_remove, Unset):
            for shopping_lists_remove_item_element in self.shopping_lists_remove:
                files.append(
                    ("shopping_lists_remove", (None, str(shopping_lists_remove_item_element).encode(), "text/plain"))
                )

        if not isinstance(self.shopping_lists_set, Unset):
            for shopping_lists_set_item_element in self.shopping_lists_set:
                files.append(
                    ("shopping_lists_set", (None, str(shopping_lists_set_item_element).encode(), "text/plain"))
                )

        if not isinstance(self.shopping_lists_remove_all, Unset):
            files.append(
                ("shopping_lists_remove_all", (None, str(self.shopping_lists_remove_all).encode(), "text/plain"))
            )

        if not isinstance(self.substitute_children, Unset):
            if isinstance(self.substitute_children, bool):
                files.append(("substitute_children", (None, str(self.substitute_children).encode(), "text/plain")))
            else:
                files.append(("substitute_children", (None, str(self.substitute_children).encode(), "text/plain")))

        if not isinstance(self.substitute_siblings, Unset):
            if isinstance(self.substitute_siblings, bool):
                files.append(("substitute_siblings", (None, str(self.substitute_siblings).encode(), "text/plain")))
            else:
                files.append(("substitute_siblings", (None, str(self.substitute_siblings).encode(), "text/plain")))

        if not isinstance(self.ignore_shopping, Unset):
            if isinstance(self.ignore_shopping, bool):
                files.append(("ignore_shopping", (None, str(self.ignore_shopping).encode(), "text/plain")))
            else:
                files.append(("ignore_shopping", (None, str(self.ignore_shopping).encode(), "text/plain")))

        if not isinstance(self.on_hand, Unset):
            if isinstance(self.on_hand, bool):
                files.append(("on_hand", (None, str(self.on_hand).encode(), "text/plain")))
            else:
                files.append(("on_hand", (None, str(self.on_hand).encode(), "text/plain")))

        if not isinstance(self.parent_remove, Unset):
            if isinstance(self.parent_remove, bool):
                files.append(("parent_remove", (None, str(self.parent_remove).encode(), "text/plain")))
            else:
                files.append(("parent_remove", (None, str(self.parent_remove).encode(), "text/plain")))

        if not isinstance(self.parent_set, Unset):
            if isinstance(self.parent_set, int):
                files.append(("parent_set", (None, str(self.parent_set).encode(), "text/plain")))
            else:
                files.append(("parent_set", (None, str(self.parent_set).encode(), "text/plain")))

        for prop_name, prop in self.additional_properties.items():
            files.append((prop_name, (None, str(prop).encode(), "text/plain")))

        return files

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        foods = cast(list[int], d.pop("foods"))

        substitute_add = cast(list[int], d.pop("substitute_add"))

        substitute_remove = cast(list[int], d.pop("substitute_remove"))

        substitute_set = cast(list[int], d.pop("substitute_set"))

        inherit_fields_add = cast(list[int], d.pop("inherit_fields_add"))

        inherit_fields_remove = cast(list[int], d.pop("inherit_fields_remove"))

        inherit_fields_set = cast(list[int], d.pop("inherit_fields_set"))

        child_inherit_fields_add = cast(list[int], d.pop("child_inherit_fields_add"))

        child_inherit_fields_remove = cast(list[int], d.pop("child_inherit_fields_remove"))

        child_inherit_fields_set = cast(list[int], d.pop("child_inherit_fields_set"))

        def _parse_category(data: object) -> int | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(int | None | Unset, data)

        category = _parse_category(d.pop("category", UNSET))

        substitute_remove_all = d.pop("substitute_remove_all", UNSET)

        inherit_fields_remove_all = d.pop("inherit_fields_remove_all", UNSET)

        child_inherit_fields_remove_all = d.pop("child_inherit_fields_remove_all", UNSET)

        shopping_lists_add = cast(list[int], d.pop("shopping_lists_add", UNSET))

        shopping_lists_remove = cast(list[int], d.pop("shopping_lists_remove", UNSET))

        shopping_lists_set = cast(list[int], d.pop("shopping_lists_set", UNSET))

        shopping_lists_remove_all = d.pop("shopping_lists_remove_all", UNSET)

        def _parse_substitute_children(data: object) -> bool | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(bool | None | Unset, data)

        substitute_children = _parse_substitute_children(d.pop("substitute_children", UNSET))

        def _parse_substitute_siblings(data: object) -> bool | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(bool | None | Unset, data)

        substitute_siblings = _parse_substitute_siblings(d.pop("substitute_siblings", UNSET))

        def _parse_ignore_shopping(data: object) -> bool | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(bool | None | Unset, data)

        ignore_shopping = _parse_ignore_shopping(d.pop("ignore_shopping", UNSET))

        def _parse_on_hand(data: object) -> bool | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(bool | None | Unset, data)

        on_hand = _parse_on_hand(d.pop("on_hand", UNSET))

        def _parse_parent_remove(data: object) -> bool | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(bool | None | Unset, data)

        parent_remove = _parse_parent_remove(d.pop("parent_remove", UNSET))

        def _parse_parent_set(data: object) -> int | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(int | None | Unset, data)

        parent_set = _parse_parent_set(d.pop("parent_set", UNSET))

        food_batch_update = cls(
            foods=foods,
            substitute_add=substitute_add,
            substitute_remove=substitute_remove,
            substitute_set=substitute_set,
            inherit_fields_add=inherit_fields_add,
            inherit_fields_remove=inherit_fields_remove,
            inherit_fields_set=inherit_fields_set,
            child_inherit_fields_add=child_inherit_fields_add,
            child_inherit_fields_remove=child_inherit_fields_remove,
            child_inherit_fields_set=child_inherit_fields_set,
            category=category,
            substitute_remove_all=substitute_remove_all,
            inherit_fields_remove_all=inherit_fields_remove_all,
            child_inherit_fields_remove_all=child_inherit_fields_remove_all,
            shopping_lists_add=shopping_lists_add,
            shopping_lists_remove=shopping_lists_remove,
            shopping_lists_set=shopping_lists_set,
            shopping_lists_remove_all=shopping_lists_remove_all,
            substitute_children=substitute_children,
            substitute_siblings=substitute_siblings,
            ignore_shopping=ignore_shopping,
            on_hand=on_hand,
            parent_remove=parent_remove,
            parent_set=parent_set,
        )

        food_batch_update.additional_properties = d
        return food_batch_update

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="FoodInheritField")


@_attrs_define
class FoodInheritField:
    """Moves `UniqueValidator`'s from the validation stage to the save stage.
    It solves the problem with nested validation for unique fields on update.

    If you want more details, you can read related issues and articles:
    https://github.com/beda-software/drf-writable-nested/issues/1
    http://www.django-rest-framework.org/api-guide/validators/#updating-nested-serializers

    Example of usage:
    ```
        class Child(models.Model):
        field = models.CharField(unique=True)


    class Parent(models.Model):
        child = models.ForeignKey('Child')


    class ChildSerializer(UniqueFieldsMixin, serializers.ModelSerializer):
        class Meta:
            model = Child


    class ParentSerializer(NestedUpdateMixin, serializers.ModelSerializer):
        child = ChildSerializer()

        class Meta:
            model = Parent
    ```

    Note: `UniqueFieldsMixin` must be applied only on the serializer
    which has unique fields.

    Note: When you are using both mixins
    (`UniqueFieldsMixin` and `NestedCreateMixin` or `NestedUpdateMixin`)
    you should put `UniqueFieldsMixin` ahead.

        Attributes:
            id (int | Unset):
            name (None | str | Unset):
            field (None | str | Unset):
    """

    id: int | Unset = UNSET
    name: None | str | Unset = UNSET
    field: None | str | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        id = self.id

        name: None | str | Unset
        if isinstance(self.name, Unset):
            name = UNSET
        else:
            name = self.name

        field: None | str | Unset
        if isinstance(self.field, Unset):
            field = UNSET
        else:
            field = self.field

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if id is not UNSET:
            field_dict["id"] = id
        if name is not UNSET:
            field_dict["name"] = name
        if field is not UNSET:
            field_dict["field"] = field

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        id = d.pop("id", UNSET)

        def _parse_name(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        name = _parse_name(d.pop("name", UNSET))

        def _parse_field(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        field = _parse_field(d.pop("field", UNSET))

        food_inherit_field = cls(
            id=id,
            name=name,
            field=field,
        )

        food_inherit_field.additional_properties = d
        return food_inherit_field

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from .. import types
from ..types import UNSET, Unset

T = TypeVar("T", bound="ShoppingList")


@_attrs_define
class ShoppingList:
    """Adds nested create feature

    Attributes:
        id (int | Unset):
        name (str | Unset):
        description (str | Unset):
        color (None | str | Unset):
    """

    id: int | Unset = UNSET
    name: str | Unset = UNSET
    description: str | Unset = UNSET
    color: None | str | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        id = self.id

        name = self.name

        description = self.description

        color: None | str | Unset
        if isinstance(self.color, Unset):
            color = UNSET
        else:
            color = self.color

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if id is not UNSET:
            field_dict["id"] = id
        if name is not UNSET:
            field_dict["name"] = name
        if description is not UNSET:
            field_dict["description"] = description
        if color is not UNSET:
            field_dict["color"] = color

        return field_dict

    def to_multipart(self) -> types.RequestFiles:
        files: types.RequestFiles = []

        if not isinstance(self.id, Unset):
            files.append(("id", (None, str(self.id).encode(), "text/plain")))

        if not isinstance(self.name, Unset):
            files.append(("name", (None, str(self.name).encode(), "text/plain")))

        if not isinstance(self.description, Unset):
            files.append(("description", (None, str(self.description).encode(), "text/plain")))

        if not isinstance(self.color, Unset):
            if isinstance(self.color, str):
                files.append(("color", (None, str(self.color).encode(), "text/plain")))
            else:
                files.append(("color", (None, str(self.color).encode(), "text/plain")))

        for prop_name, prop in self.additional_properties.items():
            files.append((prop_name, (None, str(prop).encode(), "text/plain")))

        return files

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        id = d.pop("id", UNSET)

        name = d.pop("name", UNSET)

        description = d.pop("description", UNSET)

        def _parse_color(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        color = _parse_color(d.pop("color", UNSET))

        shopping_list = cls(
            id=id,
            name=name,
            description=description,
            color=color,
        )

        shopping_list.additional_properties = d
        return shopping_list

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from .. import types

T = TypeVar("T", bound="AiImport")


@_attrs_define
class AiImport:
    """
    Attributes:
        ai_provider_id (int):
        file (None | str):
        text (None | str):
        recipe_id (None | str):
    """

    ai_provider_id: int
    file: None | str
    text: None | str
    recipe_id: None | str
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        ai_provider_id = self.ai_provider_id

        file: None | str
        file = self.file

        text: None | str
        text = self.text

        recipe_id: None | str
        recipe_id = self.recipe_id

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "ai_provider_id": ai_provider_id,
                "file": file,
                "text": text,
                "recipe_id": recipe_id,
            }
        )

        return field_dict

    def to_multipart(self) -> types.RequestFiles:
        files: types.RequestFiles = []

        files.append(("ai_provider_id", (None, str(self.ai_provider_id).encode(), "text/plain")))

        if isinstance(self.file, str):
            files.append(("file", (None, str(self.file).encode(), "text/plain")))
        else:
            files.append(("file", (None, str(self.file).encode(), "text/plain")))

        if isinstance(self.text, str):
            files.append(("text", (None, str(self.text).encode(), "text/plain")))
        else:
            files.append(("text", (None, str(self.text).encode(), "text/plain")))

        if isinstance(self.recipe_id, str):
            files.append(("recipe_id", (None, str(self.recipe_id).encode(), "text/plain")))
        else:
            files.append(("recipe_id", (None, str(self.recipe_id).encode(), "text/plain")))

        for prop_name, prop in self.additional_properties.items():
            files.append((prop_name, (None, str(prop).encode(), "text/plain")))

        return files

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        ai_provider_id = d.pop("ai_provider_id")

        def _parse_file(data: object) -> None | str:
            if data is None:
                return data
            return cast(None | str, data)

        file = _parse_file(d.pop("file"))

        def _parse_text(data: object) -> None | str:
            if data is None:
                return data
            return cast(None | str, data)

        text = _parse_text(d.pop("text"))

        def _parse_recipe_id(data: object) -> None | str:
            if data is None:
                return data
            return cast(None | str, data)

        recipe_id = _parse_recipe_id(d.pop("recipe_id"))

        ai_import = cls(
            ai_provider_id=ai_provider_id,
            file=file,
            text=text,
            recipe_id=recipe_id,
        )

        ai_import.additional_properties = d
        return ai_import

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from .. import types
from ..models.automation_type_enum import AutomationTypeEnum
from ..types import UNSET, Unset

T = TypeVar("T", bound="PatchedAutomation")


@_attrs_define
class PatchedAutomation:
    """
    Attributes:
        id (int | Unset):
        type_ (AutomationTypeEnum | Unset): * `FOOD_ALIAS` - Food Alias
            * `UNIT_ALIAS` - Unit Alias
            * `KEYWORD_ALIAS` - Keyword Alias
            * `DESCRIPTION_REPLACE` - Description Replace
            * `INSTRUCTION_REPLACE` - Instruction Replace
            * `NEVER_UNIT` - Never Unit
            * `TRANSPOSE_WORDS` - Transpose Words
            * `FOOD_REPLACE` - Food Replace
            * `UNIT_REPLACE` - Unit Replace
            * `NAME_REPLACE` - Name Replace
        name (str | Unset):
        description (None | str | Unset):
        param_1 (None | str | Unset):
        param_2 (None | str | Unset):
        param_3 (None | str | Unset):
        order (int | Unset):
        disabled (bool | Unset):
        created_by (int | Unset):
    """

    id: int | Unset = UNSET
    type_: AutomationTypeEnum | Unset = UNSET
    name: str | Unset = UNSET
    description: None | str | Unset = UNSET
    param_1: None | str | Unset = UNSET
    param_2: None | str | Unset = UNSET
    param_3: None | str | Unset = UNSET
    order: int | Unset = UNSET
    disabled: bool | Unset = UNSET
    created_by: int | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        id = self.id

        type_: str | Unset = UNSET
        if not isinstance(self.type_, Unset):
            type_ = self.type_.value

        name = self.name

        description: None | str | Unset
        if isinstance(self.description, Unset):
            description = UNSET
        else:
            description = self.description

        param_1: None | str | Unset
        if isinstance(self.param_1, Unset):
            param_1 = UNSET
        else:
            param_1 = self.param_1

        param_2: None | str | Unset
        if isinstance(self.param_2, Unset):
            param_2 = UNSET
        else:
            param_2 = self.param_2

        param_3: None | str | Unset
        if isinstance(self.param_3, Unset):
            param_3 = UNSET
        else:
            param_3 = self.param_3

        order = self.order

        disabled = self.disabled

        created_by = self.created_by

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if id is not UNSET:
            field_dict["id"] = id
        if type_ is not UNSET:
            field_dict["type"] = type_
        if name is not UNSET:
            field_dict["name"] = name
        if description is not UNSET:
            field_dict["description"] = description
        if param_1 is not UNSET:
            field_dict["param_1"] = param_1
        if param_2 is not UNSET:
            field_dict["param_2"] = param_2
        if param_3 is not UNSET:
            field_dict["param_3"] = param_3
        if order is not UNSET:
            field_dict["order"] = order
        if disabled is not UNSET:
            field_dict["disabled"] = disabled
        if created_by is not UNSET:
            field_dict["created_by"] = created_by

        return field_dict

    def to_multipart(self) -> types.RequestFiles:
        files: types.RequestFiles = []

        if not isinstance(self.id, Unset):
            files.append(("id", (None, str(self.id).encode(), "text/plain")))

        if not isinstance(self.type_, Unset):
            files.append(("type", (None, str(self.type_.value).encode(), "text/plain")))

        if not isinstance(self.name, Unset):
            files.append(("name", (None, str(self.name).encode(), "text/plain")))

        if not isinstance(self.description, Unset):
            if isinstance(self.description, str):
                files.append(("description", (None, str(self.description).encode(), "text/plain")))
            else:
                files.append(("description", (None, str(self.description).encode(), "text/plain")))

        if not isinstance(self.param_1, Unset):
            if isinstance(self.param_1, str):
                files.append(("param_1", (None, str(self.param_1).encode(), "text/plain")))
            else:
                files.append(("param_1", (None, str(self.param_1).encode(), "text/plain")))

        if not isinstance(self.param_2, Unset):
            if isinstance(self.param_2, str):
                files.append(("param_2", (None, str(self.param_2).encode(), "text/plain")))
            else:
                files.append(("param_2", (None, str(self.param_2).encode(), "text/plain")))

        if not isinstance(self.param_3, Unset):
            if isinstance(self.param_3, str):
                files.append(("param_3", (None, str(self.param_3).encode(), "text/plain")))
            else:
                files.append(("param_3", (None, str(self.param_3).encode(), "text/plain")))

        if not isinstance(self.order, Unset):
            files.append(("order", (None, str(self.order).encode(), "text/plain")))

        if not isinstance(self.disabled, Unset):
            files.append(("disabled", (None, str(self.disabled).encode(), "text/plain")))

        if not isinstance(self.created_by, Unset):
            files.append(("created_by", (None, str(self.created_by).encode(), "text/plain")))

        for prop_name, prop in self.additional_properties.items():
            files.append((prop_name, (None, str(prop).encode(), "text/plain")))

        return files

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        id = d.pop("id", UNSET)

        _type_ = d.pop("type", UNSET)
        type_: AutomationTypeEnum | Unset
        if isinstance(_type_, Unset):
            type_ = UNSET
        else:
            type_ = AutomationTypeEnum(_type_)

        name = d.pop("name", UNSET)

        def _parse_description(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        description = _parse_description(d.pop("description", UNSET))

        def _parse_param_1(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        param_1 = _parse_param_1(d.pop("param_1", UNSET))

        def _parse_param_2(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        param_2 = _parse_param_2(d.pop("param_2", UNSET))

        def _parse_param_3(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        param_3 = _parse_param_3(d.pop("param_3", UNSET))

        order = d.pop("order", UNSET)

        disabled = d.pop("disabled", UNSET)

        created_by = d.pop("created_by", UNSET)

        patched_automation = cls(
            id=id,
            type_=type_,
            name=name,
            description=description,
            param_1=param_1,
            param_2=param_2,
            param_3=param_3,
            order=order,
            disabled=disabled,
            created_by=created_by,
        )

        patched_automation.additional_properties = d
        return patched_automation

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
